"""
Sort a sample
=============
"""

# %%
# In this example we present useful methods of the :class:`~openturns.Sample` object such as marginals extraction and various sorting strategies.


# %%
import openturns as ot

# %%
# We start by defining the distribution of a regular non-biased die.
die_distribution = ot.FiniteDiscreteDistribution([[i] for i in range(1, 7)])

# %%
# We consider now an experiment with two independent dice and build the corresponding random vector :
two_dice_distribution = ot.JointDistribution([die_distribution] * 2)

# %%
# We now build a sample of size :math:`n=5` from this distribution :
n = 5
sample = two_dice_distribution.getSample(n)
print(sample)

# %%
# Useful methods
# --------------

# %%
# We have access to the marginals by providing a list of the wanted indices :

# the first marginal
sample_die1 = sample.getMarginal([0])

# the second marginal
sample_die2 = sample.getMarginal([1])

# %%
# Suppose we are interested in the sum of the two dice. This can be done by summing the two samples `die1` and `die2`.
# Provided the dimensions are the same we can add samples with the `+` operator and produce a new sample :
experiment = sample_die1 + sample_die2

# %%
# Note that the `+=` operator is defined as well.

# %%
# We can concatenate two samples having the same size with the `stack` method :
sample_die1.stack(sample_die2)
print(sample_die1)

# %%
# We can split a sample in two by giving an index (here 2).
remaining = sample_die1.split(2)
print(sample_die1)
print(remaining)


# %%
# Sorting samples
# ---------------


# %%
# We can extract any marginal and sort it by ascending order by specifying the index :
sorted_marginal = sample.sort(1)
print(sorted_marginal)

# %%
# We can sort the sample in place, that is without creating a new sample, as well with `sortInPlace`.
# When the dimension is greater than one the sort is made according to the first marginal.
sample.sortInPlace()
print(sample)

# %%
# We can sort the rows according to the second marginal with the `sortAccordingToAComponent` :
another_sample = sample.sortAccordingToAComponent(1)
print(another_sample)

# %%
# There is also a `sortAccordingToAComponentInPlace` method that does the same without creating a new sample.

# %%
# We can sort and remove the duplicates at the same time
print(sample_die2)
print(sample_die2.sortUnique())

# %%
# We note that the sample is smaller as expected. Sorting in place is also possible :
sample_die2.sortUniqueInPlace()
print(sample_die2)

# %%
# Let's try with the sample in dimension 2 :
sampleUnique = sample.sortUnique()
print(sampleUnique)

# %%
# Nothing happens here because pairs are already unique !



"""
Gaussian Process Regression : quick-start
=========================================
"""

# %%
# Abstract
# --------
#
# In this example, we create a Gaussian Process Regression (refer to :ref:`gaussian_process_regression`) for a function which has
# scalar real inputs and outputs.
# We show how to create the learning and the validation samples.
# We show how to create the surrogate model by choosing a trend and a covariance model.
# Finally, we compute the predicted confidence interval using the conditional variance.

# %%
# Introduction
# ------------
#
# We consider the sine function:
#
# .. math::
#    \model(x) = \sin(x)
#
#
# for any :math:`x\in[0,12]`.
#
# We want to create a surrogate model of this function. This is why we create a sample of :math:`n` observations of the function:
#
# .. math::
#    y_i = \model(x_i)
#
#
# for :math:`i=1,...,7`, where :math:`x_i` is the i-th input and :math:`y_i` is the corresponding output.
#
# We consider the seven following inputs :
#
# ============ === === === === ===== ==== ======
#  :math:`i`    1   2   3   4    5     6    7
# ============ === === === === ===== ==== ======
#  :math:`x_i`  1   3   4   6   7.9   11   11.5
# ============ === === === === ===== ==== ======
#
# We are going to consider a Gaussian Process Regression surrogate model with:
#
# * a constant trend,
# * a Matern covariance model.
#
import openturns as ot
import openturns.viewer as otv


# %%
# We begin by defining the function :math:`\model` as a symbolic function.
g = ot.SymbolicFunction(["x"], ["sin(x)"])

# %%
# Then we define the `x_train` variable which contains the inputs of the design of experiments of the training step.
# Then we compute the `y_train` corresponding outputs. The variable `n_train` is the size of the training design of experiments.
x_train = ot.Sample([[x] for x in [1.0, 3.0, 4.0, 6.0, 7.9, 11.0, 11.5]])
y_train = g(x_train)
n_train = x_train.getSize()
n_train

# %%
# In order to compare the function and its surrogate model, we use a test (i.e. validation) design of experiments made of a regular grid of 100 points from 0 to 12.
# Then we convert this grid into a :class:`~openturns.Sample` and we compute the outputs of the
# function on this sample.
xmin = 0.0
xmax = 12.0
n_test = 100
step = (xmax - xmin) / (n_test - 1)
myRegularGrid = ot.RegularGrid(xmin, step, n_test)
x_test = myRegularGrid.getVertices()
y_test = g(x_test)

# %%
# In order to observe the function and the location of the points in the input design of experiments, we define the following function which plots the data.


def plot_1d_data(x_data, y_data, type="Curve", legend=None, color=None, linestyle=None):
    """Plot the data (x_data,y_data) as a Cloud/Curve"""
    if type == "Curve":
        graphF = ot.Curve(x_data, y_data)
    else:
        graphF = ot.Cloud(x_data, y_data)
    if legend is not None:
        graphF.setLegend(legend)
    if color is not None:
        graphF.setColor(color)
    if linestyle is not None:
        graphF.setLineStyle(linestyle)
    return graphF


# %%
# Here, we draw the model and the train sample.
graph = ot.Graph("Model and Train sample", "X", "Y")
graph.add(
    plot_1d_data(x_test, y_test, legend="model", color="black", linestyle="dashed")
)
graph.add(
    plot_1d_data(x_train, y_train, type="Cloud", legend="train sample", color="red")
)
graph.setLegendPosition("upper right")
view = otv.View(graph)

# %%
# Creation of the surrogate model
# -------------------------------
#
# We use the :class:`~openturns.ConstantBasisFactory` class to define the trend and the
# :class:`~openturns.MaternModel` class to define the covariance model.
# In this example, the smoothness parameter of the Matérn model is fixed to :math:`\nu=3/2` and
# we only estimate the scale and the amplitude parameters.
#
# Nevertheless, we could modify the list of the
# parameters that have to be estimated (the *active* parameters) and in particular we can add the
# estimation of :math:`\nu`: see the documentation of the method
# :meth:`~openturns.CovarianceModel.setActiveParameter` of
# the class :class:`~openturns.CovarianceModel` to get more details.
dimension = 1
basis = ot.ConstantBasisFactory(dimension).build()
covarianceModel = ot.MaternModel([1.0] * dimension, 1.5)

# %%
# The class :class:`~openturns.GaussianProcessFitter`  builds the Gaussian process :math:`Y` defined by:
#
# .. math::
#
#   Y(\omega, x) = \mu(x) + W(\omega, x)
#
# where:
#
# - :math:`\mu(x) = \sum_{j=1}^{b} \beta_j \varphi_j(x)` and :math:`\varphi_j: \Rset \rightarrow \Rset`
#   the trend function for :math:`1 \leq j \leq b`. Here the functional basis is reduced to the constant
#   function;
# - :math:`W` is a Gaussian process of dimension 1 with zero mean and a Matérn covariance model
#   which covariance function is denoted by :math:`C`.
#
# The coefficients of the trend function and the active covariance model parameters are estimated by
# maximizing the *reduced* log-likelihood of the model.
fitter_algo = ot.GaussianProcessFitter(x_train, y_train, covarianceModel, basis)
fitter_algo.run()
fitter_result = fitter_algo.getResult()
print(fitter_result)

# %%
# We can draw the trend function.
trend_func = fitter_result.getMetaModel()
g_trend = trend_func.draw(xmin, xmax, 256)
g_trend.setTitle(r"Trend function of the Gaussian process $Y$")
g_trend.setXTitle(r"$x$")
g_trend.setYTitle(r"$\mu(x)$")
view = otv.View(g_trend)

# %%
# The class :class:`~openturns.GaussianProcessRegression` is built from the  Gaussian process :math:`Y` and makes
# the  Gaussian process approximation :math:`\vect{Z}` interpolate the data set and is defined as:
#
# .. math::
#    :label: GPRdefEx
#
#    \vect{Z}(\omega, \vect{x}) = \vect{Y}(\omega, \vect{x})\, | \,  \cC
#
# where :math:`\cC` is the condition :math:`\vect{Y}(\omega, \vect{x}_k) = \vect{y}_k` for
# :math:`1 \leq k \leq \sampleSize`. The Gaussian process regression surrogate model is defined by the mean of :math:`\vect{Z}`:
#
# .. math::
#
#    \metaModel(\vect{x}) = \vect{\mu}(\vect{x}) + \sum_{i=1}^\sampleSize \gamma_i \mat{C}( \vect{x},  \vect{x}_i)
#
# where the :math:`\gamma_i` are called the *covariance coefficients* and :math:`C` the covariance function of the Matérn
# covariance model.
gpr_algo = ot.GaussianProcessRegression(fitter_result)
gpr_algo.run()
gpr_result = gpr_algo.getResult()
print(gpr_result)

# %%
# We observe that the `scale` and `amplitude` parameters have been optimized by the
# :meth:`~openturns.GaussianProcessFitter.run` method, while the :math:`\nu`
# parameter has remained unchanged.
# Then we get the surrogate model with
# :meth:`~openturns.GaussianProcessFitterResult.getMetaModel` and we
# evaluate the outputs of the surrogate model on the test
# design of experiments.

# %%
gprMetamodel = gpr_result.getMetaModel()
y_test_MM = gprMetamodel(x_test)


# %%
# Now we plot Gaussian process regression surrogate model, in addition to the previous plots.
graph = ot.Graph("Gaussian process regression surrogate model", "X", "Y")
graph.add(
    plot_1d_data(x_test, y_test, legend="model", color="black", linestyle="dashed")
)
graph.add(
    plot_1d_data(x_train, y_train, type="Cloud", legend="train sample", color="red")
)
graph.add(plot_1d_data(x_test, y_test_MM, legend="GPR", color="blue"))
graph.setLegendPosition("upper right")
view = otv.View(graph)

# %%
# We observe that the Gaussian process regression surrogate model is interpolating. This is what is meant by
# *conditioning* a Gaussian process.
#
# We see that, when the sine function has a strong curvature between two points which are separated
# by a
# large distance (e.g. between :math:`x=4` and :math:`x=6`),
# then the Gaussian regression is not close to the function :math:`g`.
# However, when the training points are close (e.g. between :math:`x=11` and :math:`x=11.5`) or when the function is nearly
# linear (e.g. between :math:`x=8` and :math:`x=11`),
# then the Gaussian process regression is quite accurate.

# %%
# Compute confidence bounds
# -------------------------
#
# In order to assess the quality of the surrogate model, we can estimate the variance and compute a
# :math:`1-\alpha = 95\%` confidence interval associated with the conditioned Gaussian process.
#
# We denote by :math:`q_{p}` the quantile of order :math:`p` of the Gaussian distribution.
# Therefore, the confidence interval of level :math:`1-\alpha` is :math:`\left[q_{\alpha/2},q_{1-\alpha/2}\right]`.
alpha = 0.05


def computeQuantileAlpha(alpha):
    bilateralCI = ot.Normal().computeBilateralConfidenceInterval(1 - alpha)
    return bilateralCI.getUpperBound()[0]


quantileAlpha = computeQuantileAlpha(alpha)
print("alpha=%f" % (alpha))
print("Quantile alpha=%f" % (quantileAlpha))

# %%
# The Gaussian process regression computed on the sample :math:`(\xi_1, \dots, \xi_N)` is a Gaussian vector. It is possible to
# get the variance of each :math:`\vect{Z}_i(\omega) = \vect{Y}(\omega, \vect{\xi}_i)\, | \,  \cC` for :math:`1 \leq i \leq N`
# with
# the :meth:`~openturns.GaussianProcessConditionalCovariance.getConditionalMarginalVariance` method. That method
# returns a point which is the sequence of the variances of each :math:`\vect{Z}_i(\omega)`.
# Since this is a variance, we use the square root in order to compute the
# standard deviation.
sqrt = ot.SymbolicFunction(["x"], ["sqrt(x)"])
gccc = ot.GaussianProcessConditionalCovariance(gpr_result)
conditionalVariance = gccc.getConditionalMarginalVariance(x_test)
conditionalSigma = sqrt(conditionalVariance)

# %%
# The following figure presents the conditional standard deviation depending on :math:`x`.
graph = ot.Graph(
    "Conditional standard deviation", "x", "Conditional standard deviation", True, ""
)
curve = ot.Curve(x_test, conditionalSigma)
graph.add(curve)
view = otv.View(graph)


# %%
# We now compute the bounds of the confidence interval. For this purpose we define a small function
# `computeBoundsConfidenceInterval` :


# %%
def computeBoundsConfidenceInterval(quantileAlpha):
    dataLower = [
        [y_test_MM[i, 0] - quantileAlpha * conditionalSigma[i, 0]]
        for i in range(n_test)
    ]
    dataUpper = [
        [y_test_MM[i, 0] + quantileAlpha * conditionalSigma[i, 0]]
        for i in range(n_test)
    ]
    dataLower = ot.Sample(dataLower)
    dataUpper = ot.Sample(dataUpper)
    return dataLower, dataUpper


# %%
# We define two small lists to draw three different confidence intervals (defined by the alpha value) :
alphas = [0.05, 0.1, 0.2]
# three different green colors defined by HSV values
mycolors = [[120, 1.0, 1.0], [120, 1.0, 0.75], [120, 1.0, 0.5]]

# %%
# We are ready to display all the previous information and the three confidence intervals we want.

# %%
# sphinx_gallery_thumbnail_number = 5
graph = ot.Graph(
    "Gaussian process regression surrogate model and confidence bounds",
    "X",
    "Y",
    True,
    "",
)

# Now we loop over the different values :
for idx, v in enumerate(alphas):
    quantileAlpha = computeQuantileAlpha(v)
    vLow, vUp = computeBoundsConfidenceInterval(quantileAlpha)
    boundsPoly = ot.Polygon.FillBetween(x_test, vLow, vUp)
    boundsPoly.setColor(
        ot.Drawable.ConvertFromHSV(mycolors[idx][0], mycolors[idx][1], mycolors[idx][2])
    )
    boundsPoly.setLegend(" %d%% bounds" % ((1.0 - v) * 100))
    graph.add(boundsPoly)

graph.add(
    plot_1d_data(x_test, y_test, legend="model", color="black", linestyle="dashed")
)
graph.add(plot_1d_data(x_train, y_train, type="Cloud", legend="Data", color="red"))
graph.add(plot_1d_data(x_test, y_test_MM, legend="GPR", color="blue"))
graph.setLegendPosition("upper right")
view = otv.View(graph)

# %%
# We see that the confidence intervals are small in the regions where two
# consecutive training points are close to each other
# (e.g. between :math:`x=11` and :math:`x=11.5`) and large when the two points
# are not (e.g. between :math:`x=8.` and :math:`x=11`) or when the curvature
# of the function is large (between :math:`x=4` and :math:`x=6`).

# %%
# Display all figures.
otv.View.ShowAll()



#!/usr/bin/env python

import openturns as ot
from pathlib import Path
import re

ot.TESTPREAMBLE()

# find all keys
resourcemap_file = (
    Path(__file__)
    .parents[2]
    .joinpath("lib")
    .joinpath("src")
    .joinpath("Base")
    .joinpath("Common")
    .joinpath("ResourceMap.cxx")
)
resourcemap_lines = []
with open(resourcemap_file) as f:
    resourcemap_lines += f.read().splitlines()
resourcemap_content = {}
for line in resourcemap_lines:
    match = re.search(r'addAs(\w+)\("([\w\-]+)",[ ]*([\w\d\.\-\"]+)[\),]', line)
    if match is not None:
        key, vtype, value = match.group(2), match.group(1), match.group(3)
        print(key, vtype, value)
        if key == "SymbolicParser-Backend":
            continue
        if vtype == "Scalar":
            resourcemap_content[key] = float(value)
        elif vtype == "UnsignedInteger":
            resourcemap_content[key] = int(value)
        elif vtype == "Bool":
            resourcemap_content[key] = {"true": True, "false": False}[value]
        elif vtype.startswith("String"):
            resourcemap_content[key] = value.strip('"')
        else:
            raise ValueError(f"got {key}")
print(f"resourcemap: {len(resourcemap_content)} keys")

conf_file = (
    Path(__file__)
    .parents[2]
    .joinpath("lib")
    .joinpath("etc")
    .joinpath("openturns.conf.in")
)
conf_lines = []
with open(conf_file) as f:
    conf_lines += f.read().splitlines()
conf_content = {}
for line in conf_lines:
    match = re.search(r'^[ ]+<([\w\-]+)[ ]+value_(\w+)="([\w\d\.\-]+)"', line)
    if match is not None:
        key, vtype, value = match.group(1), match.group(2), match.group(3)
        if vtype == "float":
            conf_content[key] = float(value)
        elif vtype == "int":
            conf_content[key] = int(value)
        elif vtype == "bool":
            assert value in ["true", "false"], f"value should be true|false got {value}"
            conf_content[key] = value == "true"
        elif vtype == "str":
            conf_content[key] = value
        else:
            raise ValueError(f"got {vtype}")
print(f"config: {len(conf_content)} keys")

# check missing keys in config wrt resourcemap
for key in resourcemap_content:
    if key in conf_content:
        if resourcemap_content[key] != conf_content[key]:
            raise ValueError(
                f"key={key} RM={resourcemap_content[key]} CONF={conf_content[key]}"
            )
    else:
        raise ValueError(f"missing {key} in rm")

# check extra keys in config wrt resourcemap
for key in conf_content:
    if key not in resourcemap_content:
        raise ValueError(f"extra {key} in config")

# find all docstring files
docstring_files = Path(__file__).parents[1].joinpath("src").glob("*_doc.i")
docstring_lines = []
for docstring_file in docstring_files:
    with open(docstring_file, errors="replace") as f:
        docstring_lines += f.read().splitlines()

# count undocumented keys
undoc_keys = dict(resourcemap_content)
for key in resourcemap_content:
    for line in docstring_lines:
        if key in line:
            del undoc_keys[key]
            break

for key in undoc_keys:
    print(key)
n_undoc_keys = len(undoc_keys)
n_keys = len(resourcemap_content)
print(f"found {n_undoc_keys} undocumented keys out of {n_keys}")
assert n_undoc_keys < 475, "too much undocumented resourcemap keys"



#! /usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal
from openturns.usecases import deflection_tube


ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(5)

"""
Test the import of the DeflectionTube data class.
"""
dt = deflection_tube.DeflectionTube()

assert_almost_equal(dt.dim, 6, 1e-12)

# marginal means
assert_almost_equal(dt.XF.getMean()[0], 1.0, 1e-12)
assert_almost_equal(dt.XE.getMean()[0], 200000.0, 1e-12)
assert_almost_equal(dt.XL.getMean()[0], 1.5, 1e-12)
assert_almost_equal(dt.Xa.getMean()[0], 1.0, 1e-12)
assert_almost_equal(dt.XD.getMean()[0], 0.8, 1e-12)
assert_almost_equal(dt.Xd.getMean()[0], 0.1, 1e-12)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Test the initialization method
ot.RandomGenerator.SetSeed(0)

# Test the state accessor for reading
initialState = ot.RandomGenerator.GetState()
print("initialState=", repr(initialState))
# Test the generation of one realization
realization = ot.RandomGenerator.Generate()
print("one realization=%.6f" % realization)
state = ot.RandomGenerator.GetState()
print("state=", repr(state))
print("new state index=", state.getIndex())

# Test the generation of several realizations
size = 10000
several = ot.RandomGenerator.Generate(size)
print(size, " realizations, first=%.6f" % several[0], ", last=%.6f" % several[size - 1])

# Test the state accessor for writing
ot.RandomGenerator.SetState(initialState)
print("restoring initial state, one realization=%.6f" % ot.RandomGenerator.Generate())
# Test the uniformity of the random generator
size = 100000
manyPoints = ot.RandomGenerator.Generate(size)
slice = 10
frequencies = ot.Point(slice, 0.0)
mean = 0.0
for i in range(size):
    frequencies[int(slice * manyPoints[i])] = (
        frequencies[int(slice * manyPoints[i])] + 1
    )
    mean = mean + manyPoints[i]

mean = mean / size
print("mean of", size, "realizations= %.12f" % mean)
for i in range(slice):
    frequencies[i] = frequencies[i] / size
    print(
        "frequency in slice [",
        (1.0 * i) / slice,
        ",",
        (i + 1.0) / slice,
        "]=",
        frequencies[i],
    )

print(
    "One integer generation in [[0, ",
    slice - 1,
    "]]=",
    ot.RandomGenerator.IntegerGenerate(slice),
)
print(
    "Another integer generation in [[0, ",
    slice - 1,
    "]]=",
    ot.RandomGenerator.IntegerGenerate(slice),
)
manyUnsignedInteger = ot.RandomGenerator.IntegerGenerate(size, slice)
frequencies = ot.Point(slice, 0.0)
mean = 0.0
for i in range(size):
    frequencies[manyUnsignedInteger[i]] = frequencies[manyUnsignedInteger[i]] + 1
    mean = mean + manyUnsignedInteger[i]

mean = mean / size
print("mean of ", size, " integer realizations=", mean)
for i in range(slice):
    frequencies[i] = frequencies[i] / size
    print("frequency for value ", i, "=", frequencies[i])



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Defining parameters
dimension = 3
size = 100

# Build standard LHS algorithm
distribution = ot.JointDistribution([ot.Uniform(0.0, 1.0)] * dimension)
lhs = ot.LHSExperiment(distribution, size)
lhs.setRandomShift(False)  # centered
lhs.setAlwaysShuffle(True)  # randomized

# print the object
print("lhs=", lhs)
bounds = distribution.getRange()
print("Bounds of uniform distributions=", bounds)

# Generate design without optimization
design = lhs.generate()
print("design=", design)

# Defining space fillings
spaceFillingC2 = ot.SpaceFillingC2()
spaceFillingPhiP = ot.SpaceFillingPhiP()

# print the criteria on this design
print(
    "PhiP=%f, C2=%f"
    % (ot.SpaceFillingPhiP().evaluate(design), ot.SpaceFillingC2().evaluate(design))
)

# Parameters for drawing design ==> Number of points are for the "grid"
Nx = 50
Ny = 50

# --------------------------------------------------#
# ------------ MonteCarlo algorithm  ------------- #
# --------------------------------------------------#

# RandomBruteForce MonteCarlo with N designs
N = 100

# 1) LHS with C2 optimization
optimalLHSAlgorithm = ot.MonteCarloLHS(lhs, N, spaceFillingC2)
# print lhs
print("lhs=", optimalLHSAlgorithm)
design = optimalLHSAlgorithm.generate()
print("Generating design with MonteCarlo and C2 space filling=", design)
result = optimalLHSAlgorithm.getResult()
print("History criterion=", result.getAlgoHistory())
print("Final criteria: C2=%f, PhiP=%f" % (result.getC2(), result.getPhiP()))

# Criterion graph ==> Graph object
criterionGraph = result.drawHistoryCriterion()

# 2) LHS with PhiP optimization (=mindist optim)
optimalLHSAlgorithm = ot.MonteCarloLHS(lhs, N, spaceFillingPhiP)
print("lhs=", optimalLHSAlgorithm)
design = optimalLHSAlgorithm.generate()
print("Generating design with MonteCarlo and PhiP optimum=", design)
result = optimalLHSAlgorithm.getResult()
print("History criterion=", result.getAlgoHistory())
print("Final criteria: C2=%f, PhiP=%f" % (result.getC2(), result.getPhiP()))

# Graph of criterion
criterionGraph = result.drawHistoryCriterion()

# --------------------------------------------------#
# ------------- Simulated annealing  ------------- #
# --------------------------------------------------#
# Defining temperature profil ==> TO, iterations...
T0 = 10.0
iMax = 2000
c = 0.95
# Care, c should be in ]0,1[
# Geometric profil
geomProfile = ot.GeometricProfile(T0, c, iMax)

# 3) Simulated Annealing LHS with geometric temperature, C2 optimization
optimalLHSAlgorithm = ot.SimulatedAnnealingLHS(lhs, spaceFillingC2, geomProfile)
print("lhs=", optimalLHSAlgorithm)
design = optimalLHSAlgorithm.generate()
print(
    "Generating design using SimulatedAnnealing geometric temperature & C2 criterion=",
    design,
)
result = optimalLHSAlgorithm.getResult()
history = result.getAlgoHistory()
print("History criterion=", history[:, 0])
print("History temperature=", history[:, 1])
print("History probability=", history[:, 2])
print("Final criteria: C2=%f, PhiP=%f" % (result.getC2(), result.getPhiP()))

# Criteria drawing
# SA algorithms returns also Probability & temperature
criterionGraph = result.drawHistoryCriterion()
probaGraph = result.drawHistoryProbability()
tempGraph = result.drawHistoryTemperature()

# 4) Simulated Annealing LHS with geometric temperature, PhiP optimization
optimalLHSAlgorithm = ot.SimulatedAnnealingLHS(lhs, spaceFillingPhiP, geomProfile)
print("lhs=", optimalLHSAlgorithm)
design = optimalLHSAlgorithm.generate()
print(
    "Generating design using SimulatedAnnealing geometric temperature & C2 criterion=",
    design,
)
result = optimalLHSAlgorithm.getResult()
history = result.getAlgoHistory()
print("History criterion=", history[:, 0])
print("History temperature=", history[:, 1])
print("History probability=", history[:, 2])
print("Final criteria: C2=%f, PhiP=%f" % (result.getC2(), result.getPhiP()))

# Criteria drawing
criterionGraph = result.drawHistoryCriterion()
probaGraph = result.drawHistoryProbability()
tempGraph = result.drawHistoryTemperature()

# Linear profil
linearProfile = ot.LinearProfile(T0, iMax)

# 5) Simulated Annealing LHS with linear temperature, C2 optimization
optimalLHSAlgorithm = ot.SimulatedAnnealingLHS(lhs, spaceFillingC2, linearProfile)
print("lhs=", optimalLHSAlgorithm)
design = optimalLHSAlgorithm.generate()
print(
    "Generating design using linear temperature SimulatedAnnealing & C2 criterion =",
    design,
)
result = optimalLHSAlgorithm.getResult()
history = result.getAlgoHistory()
print("History criterion=", history[:, 0])
print("History temperature=", history[:, 1])
print("History probability=", history[:, 2])
print("Final criteria: C2=%f, PhiP=%f" % (result.getC2(), result.getPhiP()))

# Criteria drawing
criterionGraph = result.drawHistoryCriterion()
probaGraph = result.drawHistoryProbability()
tempGraph = result.drawHistoryTemperature()

# 6) Simulated Annealing LHS with linear temperature, PhiP optimization
optimalLHSAlgorithm = ot.SimulatedAnnealingLHS(lhs, spaceFillingPhiP, linearProfile)
print("lhs=", optimalLHSAlgorithm)
design = optimalLHSAlgorithm.generate()
print(
    "Generating design using linear temperature SimulatedAnnealing & PhiP criterion =",
    design,
)
result = optimalLHSAlgorithm.getResult()
history = result.getAlgoHistory()
print("History criterion=", history[:, 0])
print("History temperature=", history[:, 1])
print("History probability=", history[:, 2])
print("Final criteria: C2=%f, PhiP=%f" % (result.getC2(), result.getPhiP()))

# Criteria drawing
criterionGraph = result.drawHistoryCriterion()
probaGraph = result.drawHistoryProbability()
tempGraph = result.drawHistoryTemperature()

# 7) Simulated Annealing LHS with linear temperature, Phi optimization and nStart > 1
#  Result is a collection of LHSResult
nStart = 10
optimalDesign = optimalLHSAlgorithm.generateWithRestart(nStart)
for i in range(nStart):
    result = optimalLHSAlgorithm.getResult()
    design = result.getOptimalDesign(i)
    history = result.getAlgoHistory(i)
    print(
        "Generating design using linear temperature SimulatedAnnealing & PhiP criterion =",
        design,
    )
    print("History criterion=", history[:, 0])
    # print("History temperature=", history[:,1])
    # print("History probability=", history[:,2])
    print("Final criteria: C2=%f, PhiP=%f" % (result.getC2(i), result.getPhiP(i)))

    # decreasing criterion
    criterionGraph = result.drawHistoryCriterion(i)
    probaGraph = result.drawHistoryProbability(i)
    tempGraph = result.drawHistoryTemperature(i)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
from openturns.usecases import coles

ot.TESTPREAMBLE()
ot.Log.Show(ot.Log.INFO)

size = 10000
distribution = ot.GeneralizedExtremeValue(2.0, 1.5, -0.15)
sample = distribution.getSample(size)
factory = ot.GeneralizedExtremeValueFactory()

estimated_bic_selection = factory.build(sample)
print("Estimated distribution (BIC selection)=", repr(estimated_bic_selection))
ott.assert_almost_equal(
    estimated_bic_selection.getParameter(), distribution.getParameter(), 5e-2, 5e-2
)

default_distribution = factory.build()
print("Default distribution=", default_distribution)
assert (
    default_distribution.getParameter() == ot.GeneralizedExtremeValue().getParameter()
)

estimated_param = factory.build(distribution.getParameter())
print("Distribution from parameters=", estimated_param)
assert estimated_param.getParameter() == distribution.getParameter(), "wrong parameters"

estimated_bic_selection2 = factory.buildAsGeneralizedExtremeValue(sample)
print("Estimated GeneralizedExtremeValue (BIC selection)=", estimated_bic_selection2)
assert estimated_bic_selection2.__class__.__name__ == "GeneralizedExtremeValue"

default_distribution2 = factory.buildAsGeneralizedExtremeValue()
print("Default GeneralizedExtremeValue=", default_distribution2)
assert (
    default_distribution2.getParameter() == ot.GeneralizedExtremeValue().getParameter()
)
assert default_distribution2.__class__.__name__ == "GeneralizedExtremeValue"

estimated_param2 = factory.buildAsGeneralizedExtremeValue(distribution.getParameter())
print("GeneralizedExtremeValue from parameters=", estimated_param2)
assert (
    estimated_param2.getParameter() == distribution.getParameter()
), "wrong parameters"
assert estimated_param2.__class__.__name__ == "GeneralizedExtremeValue"

estimated_mle = factory.buildMethodOfLikelihoodMaximization(sample)
print("Estimated GeneralizedExtremeValue (MLE)=", estimated_mle)
ott.assert_almost_equal(
    estimated_mle.getParameter(), distribution.getParameter(), 1e-2, 1e-2
)

estimated_prof_mle = factory.buildMethodOfXiProfileLikelihood(sample, 1)
print("Estimated GeneralizedExtremeValue (profile MLE)=", estimated_prof_mle)
ott.assert_almost_equal(
    estimated_mle.getParameter(), distribution.getParameter(), 1e-2, 1e-2
)

# try MLE on multiple parameters
pdist = ot.JointDistribution(
    [
        ot.Normal(0.0, 2.0),
        ot.TruncatedNormal(1.0, 2.0, 1e-6, 10.0),
        ot.Uniform(-0.9, 0.9),
    ]
)
for p in pdist.getSample(10):
    distribution = factory.buildAsGeneralizedExtremeValue(p)
    print("distribution=", repr(distribution))

    sample_p = distribution.getSample(size)

    estimated_mle = factory.buildMethodOfLikelihoodMaximization(sample_p)
    print("Estimated GeneralizedExtremeValue (MLE)=", estimated_mle)
    mu, sigma, xi = estimated_mle.getParameter()
    ott.assert_almost_equal(mu, distribution.getMu(), 5e-2, 2e-1)
    ott.assert_almost_equal(sigma, distribution.getSigma(), 1e-2, 5e-2)
    ott.assert_almost_equal(xi, distribution.getXi(), 1e-2, 5e-2)

# check parameters distribution is gaussian
estimator = factory.buildEstimator(sample)
print(estimator)
assert (
    estimator.getParameterDistribution().getImplementation().__class__.__name__
    == "Normal"
)

estimator_prof_mle = factory.buildMethodOfXiProfileLikelihoodEstimator(sample)
assert (
    estimator_prof_mle.getParameterDistribution().getImplementation().__class__.__name__
    == "Normal"
)

# specific check for profile likelihood
xi = estimator_prof_mle.getParameter()
try:
    ci = estimator_prof_mle.getParameterConfidenceInterval()
    print("profile MLE estimator xi=", xi, ci)
    assert [xi] in ci, "xi should be inside confidence interval"
except Exception as exception:
    print(exception)

graph = estimator_prof_mle.drawProfileLikelihoodFunction()

# specific check for R maxima
sample_rmax = coles.Coles().venice[:, 1:]
print("sample_rmax=", sample_rmax)
estimated_rmax = factory.buildMethodOfLikelihoodMaximization(sample_rmax)
print("Estimated GeneralizedExtremeValue (R maxima)=", estimated_rmax)
# These reference values give a better likelihood than the ones given by Coles
ott.assert_almost_equal(
    estimated_rmax.getParameter(), [116.868, 11.8544, -0.107984], 1e-2, 1e-2
)

estimator_rmax = factory.buildMethodOfLikelihoodMaximizationEstimator(sample_rmax)
assert (
    estimator_rmax.getParameterDistribution().getImplementation().__class__.__name__
    == "Normal"
)

# specific check for covariates
fremantle = coles.Coles().fremantle
timeStamps = fremantle[:, 0]
muIndices = [0]  # linear
sigmaIndices = []  # stationary
xiIndices = []  # stationary
muLink = ot.SymbolicFunction(["x"], ["1.0*x"])
estimator_covariate = factory.buildCovariates(
    fremantle[:, 1], timeStamps, muIndices, sigmaIndices, xiIndices, muLink
)
beta = estimator_covariate.getOptimalParameter()
print("beta*=", beta)
ott.assert_almost_equal(beta, [0.00203333, -2.4751, 0.124301, -0.125008], 1e-2, 1e-2)
beta_dist = estimator_covariate.getParameterDistribution()
print("beta dist=", beta_dist)
assert (
    beta_dist.getImplementation().__class__.__name__ == "Normal"
), "wrong distribution"
cov_ref = [
    [3.11388e-09, -6.05561e-06, -1.99653e-08, 2.10413e-07],
    [-6.05561e-06, 0.0117791, 3.93182e-05, -0.000414466],
    [-1.99653e-08, 3.93182e-05, 1.51924e-06, -6.06597e-06],
    [2.10413e-07, -0.000414466, -6.06597e-06, 8.53944e-05],
]
ott.assert_almost_equal(
    ot.Matrix(beta_dist.getCovariance()), ot.Matrix(cov_ref), 1e-3, 1e-3
)
graph_mu1d = estimator_covariate.drawParameterFunction1D(0)
graph_mu2d = estimator_covariate.drawParameterFunction2D(0)
graph_q_mu1d = estimator_covariate.drawQuantileFunction1D(0.9)
graph_q_mu2d = estimator_covariate.drawQuantileFunction2D(0.9)

# functional
t0 = fremantle[0, 0]  # year of first record
constant = ot.SymbolicFunction(["t"], ["1.0"])
basis = ot.Basis([constant, ot.SymbolicFunction(["t"], ["t"])])
muIndices = [0, 1]  # linear
sigmaIndices = [0]  # stationary
xiIndices = [0]  # stationary
estimator_timevar = factory.buildTimeVarying(
    fremantle[:, 1], timeStamps, basis, muIndices, sigmaIndices, xiIndices
)
beta = estimator_timevar.getOptimalParameter()
print("beta*=", beta)
ott.assert_almost_equal(beta, [1.38216, 0.187033, 0.124317, -0.125086], 1e-2, 1e-2)
beta_dist = estimator_timevar.getParameterDistribution()
print("beta dist=", beta_dist)
assert (
    beta_dist.getImplementation().__class__.__name__ == "Normal"
), "wrong distribution"
cov_ref = [
    [9.49362e-06, -1.34867e-05, 1.43649e-06, -1.51103e-05],
    [-1.34867e-05, 2.65117e-05, -1.85843e-06, 1.93419e-05],
    [1.43649e-06, -1.85843e-06, 1.52237e-06, -6.07642e-06],
    [-1.51103e-05, 1.93419e-05, -6.07642e-06, 8.52864e-05],
]
ott.assert_almost_equal(
    ot.Matrix(beta_dist.getCovariance()), ot.Matrix(cov_ref), 1e-3, 1e-3
)
dist0 = estimator_timevar.getDistribution(t0)
print(dist0)
assert dist0.getImplementation().__class__.__name__ == "GeneralizedExtremeValue"
graph_param = estimator_timevar.drawParameterFunction(0)
graph_quantile = estimator_timevar.drawQuantileFunction(0.99)

# specific check for model selection
estimator_mle = factory.buildMethodOfLikelihoodMaximizationEstimator(fremantle[:, 1])
result_deviance = ot.HypothesisTest.LikelihoodRatioTest(
    3, estimator_mle.getLogLikelihood(), 4, estimator_timevar.getLogLikelihood()
)
print(result_deviance)
ott.assert_almost_equal(result_deviance.getStatistic(), 12.674, 1e-2, 1e-2)
assert not result_deviance.getBinaryQualityMeasure(), "H0 (stationary model) accepted"

# specific check for return level
zm = factory.buildReturnLevelEstimator(estimator_mle, 10.0)
print("zm=", zm)
ott.assert_almost_equal(zm, ot.Normal(1.73377, 0.0228792), 1e-2, 1e-2)

# specific check for return level via profile likelihood
estimator_prof_rl = factory.buildReturnLevelProfileLikelihoodEstimator(
    fremantle[:, 1], 10.0
)
print(estimator_prof_rl)
zm = estimator_prof_rl.getParameter()
try:
    ci = estimator_prof_rl.getParameterConfidenceInterval()
    print("profile return level estimator zm=", zm, ci)
    assert [zm] in ci, "zm should be inside confidence interval"
except Exception as exception:
    print(exception)
graph = estimator_prof_rl.drawProfileLikelihoodFunction()



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv


distX = ot.Normal(0.0, 10.0)
myFunc = ot.SymbolicFunction(["x"], ["x+sin(x)"])
distFin = ot.CompositeDistribution(myFunc, distX)

graphPDF = distFin.drawPDF(1024)
graphPDF.setXTitle("x")
graphPDF.setLegendPosition("")
graphCDF = distFin.drawCDF(1024)
graphCDF.setXTitle("x")
graphCDF.setLegendPosition("")


fig = plt.figure(figsize=(8, 4))

pdf_axis = fig.add_subplot(121)
cdf_axis = fig.add_subplot(122)
pdf_axis.set_xlim(auto=True)
cdf_axis.set_xlim(auto=True)

otv.View(graphPDF, figure=fig, axes=[pdf_axis], add_legend=True)
otv.View(graphCDF, figure=fig, axes=[cdf_axis], add_legend=True)
fig.suptitle("CompositeDistribution: f(x)=x+sin(x); L=Normal(0.0, 10.0): pdf and cdf")



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# First : an n by n case
matrix1 = ot.Matrix(2, 2)
matrix1[0, 0] = 1.0
matrix1[1, 0] = 2.0
matrix1[0, 1] = 5.0
matrix1[1, 1] = 12.0
print("matrix1=" + repr(matrix1))

pt1 = ot.Point()
pt1.add(5.0)
pt1.add(0.0)
print("pt1=" + repr(pt1))

result1 = ot.Point()
result1 = matrix1.solveLinearSystem(pt1)
print("result1=" + repr(result1))

# Second : an n by p case, n < p
matrix2 = ot.Matrix(2, 3)
matrix2[0, 0] = 1.0
matrix2[1, 0] = 2.0
matrix2[0, 1] = 5.0
matrix2[1, 1] = 12.0
matrix2[0, 2] = 3.0
matrix2[1, 2] = 4.0
print("matrix2=" + repr(matrix2))

pt2 = ot.Point()
pt2.add(5.0)
pt2.add(0.0)
print("pt2=" + repr(pt2))

result2 = ot.Point()
result2 = matrix2.solveLinearSystem(pt2)
print("result2=" + repr(result2))

# Third : an n by p case, n > p
matrix3 = ot.Matrix(3, 2)
matrix3[0, 0] = 1.0
matrix3[1, 0] = 2.0
matrix3[2, 0] = 4.0
matrix3[0, 1] = 5.0
matrix3[1, 1] = 12.0
matrix3[2, 1] = 3.0

print("matrix3=" + repr(matrix3))

pt3 = ot.Point()
pt3.add(5.0)
pt3.add(0.0)
pt3.add(1.0)
print("pt3=" + repr(pt3))

result3 = ot.Point()
result3 = matrix3.solveLinearSystem(pt3)
print("result3=" + repr(result3))

b1 = ot.Matrix(2, 4)
b1[0, 0] = 5.0
b1[1, 0] = 1.0
b1[0, 1] = 10.0
b1[1, 1] = 2.0
b1[0, 2] = 15.0
b1[1, 2] = 3.0
b1[0, 3] = 20.0
b1[1, 3] = 4.0
print("b1=" + repr(b1))
result4 = matrix1.solveLinearSystem(b1)
print("result4=" + repr(result4))
result4 = matrix1.solveLinearSystemInPlace(b1)
print("result4=" + repr(result4))

result5 = matrix2.solveLinearSystem(b1)
print("result5=" + repr(result5))
result5 = matrix2.solveLinearSystemInPlace(b1)
print("result5=" + repr(result5))

b2 = ot.Matrix(3, 4)
b2[0, 0] = 5.0
b2[1, 0] = 1.0
b2[2, 0] = -2.0
b2[0, 1] = 10.0
b2[1, 1] = 2.0
b2[2, 1] = -4.0
b2[0, 2] = 15.0
b2[1, 2] = 3.0
b2[2, 2] = -6.0
b2[0, 3] = 20.0
b2[1, 3] = 4.0
b2[2, 3] = -8.0
print("b2=" + repr(b2))
result6 = matrix3.solveLinearSystem(b2)
print("result6=" + repr(result6))
result6 = matrix3.solveLinearSystemInPlace(b2)
print("result6 = " + repr(result6))



"""
OpenTURNS, uncertainty engineering software.

Documentation is available online at http://www.openturns.org.

Available submodules
--------------------
common                                --- Utility classes
typ                                   --- Internal data types
statistics                            --- Statistical classes
graph                                 --- Graphical output
geom                                  --- Geometrical primitives
func                                  --- Function primitives
diff                                  --- Differential algorithms
optim                                 --- Optimization routines
solver                                --- Solvers
algo                                  --- Approximation algorithms
experiment                            --- Non probabilistic designs of experiments
base                                  --- Deterministic meta-package
classification                        --- Classification algorithms
model_copula                          --- Copulas
randomvector                          --- Random vectors
dist                                  --- Probabilistic distributions
model_process                         --- Stochastic processes
weightedexperiment                    --- Probabilistic designs of experiments
orthogonalbasis                       --- Polynomial primitives
metamodel                             --- Meta-modelling
transformation                        --- Iso-probabilistic and process transformations
analytical                            --- Analytical uncertainty propagation algorithms
simulation                            --- Simulation uncertainty propagation algorithms
stattests                             --- Statistical tests
bayesian                              --- Bayesian updating
uncertainty                           --- Probabilistic meta-package

Utility modules
---------------
viewer                                --- Matplotlib-based viewer
coupling_tools                        --- External code coupling helpers

Environment variables
---------------------
OPENTURNS_HOME                        --- Custom installation path
OPENTURNS_CONFIG_PATH                 --- Path to configuration file
OPENTURNS_LOG_SEVERITY                --- Log severity flag, for example "ALL"
"""

# flake8: noqa

from .common import *
from .typ import *
from .graph import *
from .geom import *
from .func import *
from .func2 import *
from . import SpecFunc
from .statistics import *
from .diff import *
from .optim import *
from .solver import *
from .algo import *
from .experiment import *
from .base import *
from .iterative_statistics import *

from .model_copula import *
from .model_copula2 import *
from .randomvector import *
from . import DistFunc
from .dist import *
from .model_process import *
from .weightedexperiment import *
from .classification import *
from .orthogonalbasis import *
from .metamodel import *
from .transformation import *
from .analytical import *
from .simulation import *
from .stattests import *
from . import FittingTest
from . import HypothesisTest
from . import LinearModelTest
from . import NormalityTest
from . import VisualTest
from .bayesian import *
from .uncertainty import *


def Show(graph):
    """
    Display a graph pop-up.

    Parameters
    ----------
    graph : Graph
        the graph to display
    """
    from .viewer import View

    view = View(graph)
    view.ShowAll()


__version__ = PlatformInfo.GetVersion()



#! /usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal

ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(5)


def matrixToSample(matrix):
    """Converts a matrix into a Sample, so that we can
    assert_almost_equal a Matrix"""
    size = matrix.getNbRows()
    dimension = matrix.getNbColumns()
    sample = ot.Sample(size, dimension)
    for i in range(size):
        for j in range(dimension):
            sample[i, j] = matrix[i, j]
    return sample


def local2globalCovariance(localErrorCovariance, size, outputDimension):
    """
    Converts a local covariance into a global covariance.
    size = inputObservations.getSize()
    outputDimension = modelObservations.getDimension()
    """
    covarianceDimension = size * outputDimension
    observationDimension = localErrorCovariance.getDimension()
    globalErrorCovariance = ot.CovarianceMatrix(covarianceDimension)
    for i in range(size):
        for j in range(observationDimension):
            for k in range(observationDimension):
                globalErrorCovariance[
                    i * observationDimension + j, i * observationDimension + k
                ] = localErrorCovariance[j, k]
    return globalErrorCovariance


"""
Thanks to - Antoine Dumas, Phimeca
References
- J. Lemaitre and J. L. Chaboche (2002) "Mechanics of solid materials" Cambridge University Press.
"""
ot.RandomGenerator.SetSeed(0)

# Create the function.
g = ot.SymbolicFunction(["strain", "R", "C", "gam"], ["R + C*(1-exp(-gam*strain))"])

# Define the random vector.
Strain = ot.Uniform(0, 0.07)
unknownR = 750
unknownC = 2750
unknownGamma = 10
R = ot.Dirac(unknownR)
C = ot.Dirac(unknownC)
Gamma = ot.Dirac(unknownGamma)

Strain.setDescription(["Strain"])
R.setDescription(["R"])
C.setDescription(["C"])
Gamma.setDescription(["Gamma"])

# Create the joint input distribution function.
inputRandomVector = ot.JointDistribution([Strain, R, C, Gamma])

# Create the Monte-Carlo sample.
sampleSize = 10
inputSample = inputRandomVector.getSample(sampleSize)
outputStress = g(inputSample)

# Generate observation noise.
stressObservationNoiseSigma = 40.0  # (Pa)
noiseSigma = ot.Normal(0.0, stressObservationNoiseSigma)
sampleNoiseH = noiseSigma.getSample(sampleSize)
observedStress = outputStress + sampleNoiseH
observedStrain = inputSample[:, 0]

# Set the calibration parameters
# Define the value of the reference values of the $\theta$ parameter.
# In the bayesian framework, this is called the mean of the *prior* gaussian distribution.
# In the data assimilation framework, this is called the *background*.
R = 700  # Exact : 750
C = 2500  # Exact : 2750
Gamma = 8.0  # Exact : 10
thetaPrior = ot.Point([R, C, Gamma])

# The following statement create the calibrated function from the model.
# The calibrated parameters Ks, Zv, Zm are at indices 1, 2, 3 in the inputs arguments of the model.
calibratedIndices = [1, 2, 3]
mycf = ot.ParametricFunction(g, calibratedIndices, thetaPrior)

# Gaussian linear calibration
# The standard deviation of the observations.
sigmaStress = 10.0  # (Pa)

# Define the covariance matrix of the output Y of the model.
localErrorCovariance = ot.CovarianceMatrix(1)
localErrorCovariance[0, 0] = sigmaStress**2

# Define the covariance matrix of the parameters $\theta$ to calibrate.
sigmaR = 0.1 * R
sigmaC = 0.1 * C
sigmaGamma = 0.1 * Gamma

priorCovariance = ot.CovarianceMatrix(3)
priorCovariance[0, 0] = sigmaR**2
priorCovariance[1, 1] = sigmaC**2
priorCovariance[2, 2] = sigmaGamma**2

methods = ["SVD", "QR", "Cholesky"]
for method in methods:
    print("method=", method)
    # 1. Local calibration
    # The `GaussianLinearCalibration` class performs the gaussian linear
    # calibration by linearizing the model in the neighbourhood of the prior.
    algo = ot.GaussianLinearCalibration(
        mycf,
        observedStrain,
        observedStress,
        thetaPrior,
        priorCovariance,
        localErrorCovariance,
        method,
    )

    # The `run` method computes the solution of the problem.
    algo.run()

    calibrationResult = algo.getResult()

    # Analysis of the results
    # Maximum A Posteriori estimator
    thetaMAP = calibrationResult.getParameterMAP()
    exactTheta = ot.Point([762.661, 3056.59, 8.52781])
    assert_almost_equal(thetaMAP, exactTheta)

    # Covariance matrix of theta
    thetaPosterior = calibrationResult.getParameterPosterior()
    covarianceThetaStar = matrixToSample(thetaPosterior.getCovariance())
    exactCovarianceTheta = ot.Sample(
        [
            [42.4899, 288.43, -1.70502],
            [288.43, 15977.1, -67.6046],
            [-1.70502, -67.6046, 0.294659],
        ]
    )
    assert_almost_equal(covarianceThetaStar, exactCovarianceTheta)

    print("result=", calibrationResult)

    # 2. Global covariance
    outputDimension = observedStress.getDimension()
    globalErrorCovariance = local2globalCovariance(
        localErrorCovariance, sampleSize, outputDimension
    )

    # The `GaussianLinearCalibration` class performs the gaussian linear
    # calibration by linearizing the model in the neighbourhood of the prior.
    algo = ot.GaussianLinearCalibration(
        mycf,
        observedStrain,
        observedStress,
        thetaPrior,
        priorCovariance,
        globalErrorCovariance,
        method,
    )

    # The `run` method computes the solution of the problem.
    algo.run()

    # Check other fields
    calibrationResult = algo.getResult()

    # Analysis of the results
    # Maximum A Posteriori estimator
    thetaMAP = calibrationResult.getParameterMAP()
    exactTheta = ot.Point([762.661, 3056.59, 8.52781])
    assert_almost_equal(thetaMAP, exactTheta)

    # Covariance matrix of theta
    thetaPosterior = calibrationResult.getParameterPosterior()
    covarianceThetaStar = matrixToSample(thetaPosterior.getCovariance())
    exactCovarianceTheta = ot.Sample(
        [
            [42.4899, 288.43, -1.70502],
            [288.43, 15977.1, -67.6046],
            [-1.70502, -67.6046, 0.294659],
        ]
    )
    assert_almost_equal(covarianceThetaStar, exactCovarianceTheta)

    # Check other fields
    print("result=", calibrationResult)



#! /usr/bin/env python

import openturns as ot
from openturns.usecases import coles

ot.TESTPREAMBLE()

data = coles.Coles().portpirie
print(data.getSize(), data.getDimension())
assert data.getSize() == 65
assert data.getDimension() == 2

data = coles.Coles().fremantle
print(data.getSize(), data.getDimension())
assert data.getSize() == 86
assert data.getDimension() == 3

data = coles.Coles().venice
print(data.getSize(), data.getDimension())
assert data.getSize() == 50
assert data.getDimension() == 11



"""
Create your own distribution given its quantile function
========================================================
"""

# %%

# %%
# We want to create a distribution with CDF :math:`F` from the quantile function :math:`F^{-1}`.
# In order to implement this, we use the :class:`~openturns.CompositeDistribution` class.

# %%
# We know that the random variable :math:`X` is distributed according to :math:`F`
# if and only if :math:`U=F(X)` is distributed according to the uniform
# distribution in the :math:`[0,1]` interval, i.e., :math:`U=F(X) \sim \mathcal{U}(0,1)`.
# Hence, if :math:`U \sim  \mathcal{U}(0,1)` then :math:`X=F^{-1}(U)` is distributed according to :math:`F`.
#
# In this example, we want to create a distribution with CDF :math:`F: \mathbb{R} \rightarrow [0,1]` parametrized by :math:`\rho > 1`:
#
# .. math::
#    F(x) = 1-e^{-\rho^x} \quad \forall x  \in \mathbb{R}.
#
#
# The quantile function is :math:`F^{-1} : u \rightarrow [0,1]` and writes:
#
# .. math::
#    F^{-1}(u) = \dfrac{\log \left[ - \log (1-u) \right] }{\log(\rho)} \quad \forall u  \in [0,1]
#
#
# Since :math:`U \sim \mathcal{U}(0,1)`, then :math:`(1-U)\sim\mathcal{U}(0,1)`.
# This is why we can simplify the expression and define the function :math:`G` such as:
#
# .. math::
#    G(u) = \dfrac{\log \left[ - \log u \right] }{\log(\rho)} \quad \forall u  \in [0,1].
#
#
# Then :math:`G(U)` is distributed according to the :math:`F` distribution.

# %%
# First, we import the useful libraries and we create the symbolic function :math:`G`.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Then, we create the :math:`G` function with :math:`\rho = 2.0`.
# To do this, we create a function which takes both :math:`y` and :math:`\rho` as inputs and returns :math:`G(u)`.
# Then the `g` function is defined as a :class:`~openturns.ParametricFunction` with a fixed value of :math:`\rho`.

# %%
gWithParameter = ot.SymbolicFunction(["u", "rho"], ["log(-log(u)) / log(rho)"])
rho = 2.0
g = ot.ParametricFunction(gWithParameter, [1], [rho])

# %%
# We define the distribution `distF` as the image through :math:`G` of the :math:`\mathcal{U}(0,1)` distribution:

# %%
distF = ot.CompositeDistribution(g, ot.Uniform(0.0, 1.0))

# %%
# Now, we can draw its PDF, CDF and generate samples.

# %%
g = distF.drawPDF()
g.setTitle("A distribution based on the quantile function.")
g.setLegendPosition("")
view = otv.View(g)
view.ShowAll()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

inputDimension = 2

levels = [8, 6]
box = ot.Box(levels)
inputSample = box.generate()
inputSample *= 10.0

model = ot.SymbolicFunction(["x", "y"], ["cos(0.5*x) + sin(y)"])
outputSample = model(inputSample)

# Validation
sampleSize = 10
inputValidSample = ot.JointDistribution(2 * [ot.Uniform(1.0, 9.0)]).getSample(
    sampleSize
)
outputValidSample = model(inputValidSample)

# Reimplement the squared exponential covariance model
rho = ot.SymbolicFunction(["x", "y"], ["exp(-0.5* (x * x + y * y))"])
covarianceModel = ot.StationaryFunctionalCovarianceModel([6.0, 2.0], [1.5], rho)

# Basis definition
basis = ot.LinearBasisFactory(inputDimension).build()

# Kriging algorithm
algo = ot.KrigingAlgorithm(inputSample, outputSample, covarianceModel, basis)
start = [50.0] * inputDimension
loglikelihood = algo.getReducedLogLikelihoodFunction()(start)
algo.setOptimizeParameters(False)
algo.run()
result = algo.getResult()
metaModel = result.getMetaModel()
variance = result.getConditionalMarginalVariance(inputSample)
ott.assert_almost_equal(variance, ot.Sample(inputSample.getSize(), 1), 1e-14, 1e-14)


# Consistency check: does the reimplementation fit the SquaredExponential class?
squaredExponential = ot.SquaredExponential(inputDimension)
squaredExponential.setParameter([6.0, 2.0, 1.5])
algoSE = ot.KrigingAlgorithm(inputSample, outputSample, squaredExponential, basis)
loglikelihoodSE = algoSE.getReducedLogLikelihoodFunction()(start)
ott.assert_almost_equal(loglikelihood, loglikelihoodSE, 1e-8, 1e-8)

# High level consistency check: does the prediction fit too?
algoSE.setOptimizeParameters(False)
algoSE.run()
resultSE = algoSE.getResult()
metaModelSE = resultSE.getMetaModel()
ott.assert_almost_equal(
    metaModel(inputValidSample), metaModelSE(inputValidSample), 1e-8, 1e-8
)

# Validate the metamodel
ott.assert_almost_equal(outputValidSample, metaModel(inputValidSample), 5.0e-3, 5.0e-3)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

#
# Physical model
#
limitState = ot.SymbolicFunction(["r", "s"], ["r - s^2"])
dim = limitState.getInputDimension()

#
# Probabilistic model
#
mean = [11.0, 1.5]
sigma = [1.0, 0.5]
R = ot.CorrelationMatrix(dim)
myDistribution = ot.Normal(mean, sigma, R)

#
# Limit state
#
vect = ot.RandomVector(myDistribution)
output = ot.CompositeRandomVector(limitState, vect)
myEvent = ot.ThresholdEvent(output, ot.Less(), 0.0)

#
# FORM/SORM Cobyla
myCobyla = ot.Cobyla()
myCobyla.setMaximumCallsNumber(1000 * dim)
myCobyla.setMaximumAbsoluteError(1.0e-4)
myCobyla.setMaximumRelativeError(1.0e-4)
myCobyla.setMaximumResidualError(1.0e-4)
myCobyla.setMaximumConstraintError(1.0e-4)
myCobyla.setStartingPoint(myDistribution.getMean())

myAlgoC = ot.FORM(myCobyla, myEvent)
myAlgoC2 = ot.SORM(myCobyla, myEvent)

myAlgoC.run()
myAlgoC2.run()

resultC = myAlgoC.getResult()
resultC2 = myAlgoC2.getResult()

#
# FORM/SORM Abdo Rackwitz
myAbdoRackwitz = ot.AbdoRackwitz()
myAbdoRackwitz.setMaximumIterationNumber(1000)
myAbdoRackwitz.setMaximumAbsoluteError(1.0e-4)
myAbdoRackwitz.setMaximumRelativeError(1.0e-4)
myAbdoRackwitz.setMaximumResidualError(1.0e-4)
myAbdoRackwitz.setMaximumConstraintError(1.0e-4)
myAbdoRackwitz.setStartingPoint(myDistribution.getMean())

myAlgoAR = ot.FORM(myAbdoRackwitz, myEvent)
myAlgoAR2 = ot.SORM(myAbdoRackwitz, myEvent)

myAlgoAR.run()
myAlgoAR2.run()

resultAR = myAlgoAR.getResult()
resultAR2 = myAlgoAR2.getResult()

#
# Monte Carlo
CoV_MC = 0.1
experiment = ot.MonteCarloExperiment()
myMC = ot.ProbabilitySimulationAlgorithm(myEvent, experiment)
myMC.setMaximumOuterSampling(1000000)
myMC.setBlockSize(1000)
myMC.setMaximumCoefficientOfVariation(CoV_MC)
myMC.run()
result = myMC.getResult()
ott.assert_almost_equal(result.getProbabilityEstimate(), 0.000258312)

#
# LHS
CoV_LHS = 0.1
experiment = ot.LHSExperiment()
experiment.setAlwaysShuffle(True)
myLHS = ot.ProbabilitySimulationAlgorithm(myEvent, experiment)
myLHS.setMaximumOuterSampling(1000000)
myLHS.setBlockSize(100)
myLHS.setMaximumCoefficientOfVariation(CoV_LHS)
myLHS.run()
result = myMC.getResult()
ott.assert_almost_equal(result.getProbabilityEstimate(), 0.000258312)



import openturns as ot
import openturns.viewer as otv

A = ot.Matrix(2, 1)
A[0, 0] = 1.0
A[1, 0] = 2.0
const = [1.5]
center = [2.0, 3.0]
func = ot.LinearEvaluation(center, const, A)

graph = func.draw([0.0, 0.0], [4.0, 4.0], [64] * 2)
graph.setTitle(r"Linear function $(x,y) \mapsto (x-2) + 2(y-3) + 1.5$")
graph.setXTitle(r"$x$")
graph.setYTitle(r"$y$")
otv.View(graph, figure_kw={"figsize": (10, 4)}, add_legend=True)



"""
Use case: Coles
===============
"""

import os
import openturns as ot


class Coles:
    """
    Data sets for the examples from [coles2001]_.

    Attributes
    ----------
    dowjones : str
        Dow Jones Index dataset path
    fremantle : :class:`~openturns.Sample`
        Sea levels dataset
    portpirie : :class:`~openturns.Sample`
        Sea levels dataset
    racetime : :class:`~openturns.Sample`
        Race time dataset
    rain : :class:`~openturns.Sample`
        Daily rainfall dataset
    venice : :class:`~openturns.Sample`
        Sea levels dataset
    wavesurge : :class:`~openturns.Sample`
        Wave and surge heights dataset
    wind : :class:`~openturns.Sample`
        Wind speeds dataset
    wooster : str
        Temperatures dataset path

    Examples
    --------
    >>> from openturns.usecases import coles
    >>> data = coles.Coles().portpirie
    >>> print(data[:3])
    """

    def __getattr__(self, name):
        if name not in [
            "dowjones",
            "portpirie",
            "venice",
            "racetime",
            "fremantle",
            "rain",
            "wavesurge",
            "wind",
            "wooster",
        ]:
            raise AttributeError(f"'Coles' object has no attribute '{name}'")
        current_dir = os.path.dirname(os.path.abspath(__file__))
        fn = os.path.join(current_dir, name + ".csv")
        if name in ["wooster", "dowjones"]:
            # cannot read sample with date column
            dataset = fn
        else:
            dataset = ot.Sample.ImportFromCSVFile(fn, ",")
        if name == "venice":
            # strip nans
            dataset.erase(4, 5)
        return dataset



"""
Simulate an Event
=================
"""

# %%
# In this example we are going to define an Event from a scalar variable :math:`Y` in the form:
#
# .. math::
#    Y > T
#
# with :math:`T` a scalar threshold
#

# %%
import openturns as ot


# %%
# Create model f(x) = x1 + 2*x2
model = ot.SymbolicFunction(["x1", "x2"], ["x1+2*x2"])

# Create the input distribution and random vector X
inputDist = ot.Normal(2)
inputDist.setDescription(["X1", "X2"])

inputVector = ot.RandomVector(inputDist)

# Create the output random vector Y=f(X)
outputVector = ot.CompositeRandomVector(model, inputVector)

# %%
# Create the event Y > 3
threshold = 3.0
event = ot.ThresholdEvent(outputVector, ot.Greater(), threshold)

# %%
# Realization as a Bernoulli
print("realization=", event.getRealization())

# %%
# Sample of 10 realizations as a Bernoulli
print("sample=", event.getSample(10))

# %%
# Build a standard event based on an event
standardEvent = ot.StandardEvent(event)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()
ot.RandomGenerator.SetSeed(0)

nu = 4.5
dim = 5
distribution = ot.Student(nu, dim)
transformation = ot.InverseRosenblattEvaluation(distribution)
print("transformation=", repr(transformation))
point = ot.Point(dim, 0.75)
print("transformation(", point, ")=", repr(transformation(point)))
print(
    "transformation parameters gradient=", repr(transformation.parameterGradient(point))
)
print("input dimension=", transformation.getInputDimension())
print("output dimension=", transformation.getOutputDimension())



"""
Plot Smolyak multi-indices
==========================
"""

# %%
# The goal of this example is to plot the multi-indices used in Smolyak's
# quadrature.
#
# For a given dimension :math:`d_x \in \mathbb{N}` and a given level
# :math:`\ell \in \mathbb{N}`
# Smolyak's quadrature is the combination of tensorized univariate quadratures.
# These quadrature are defined by the set of multi-indices:
#
# .. math::
#
#     \mathcal{S}_{\ell, d_x} = \left\{\|\boldsymbol{k}\|_1 \leq \ell
#                                      + d_x - 1\right\}
#
# where :math:`\|\boldsymbol{k}\|_1 = k_1 + ... + k_{d_x}` is the 1-norm of the
# multi-index :math:`\boldsymbol{k} \in \mathbb{N}^{d_x}`.
#
# The goal of this script is to plot the multi-indices involved in Smolyak's
# quadrature for different values of the level :math:`\ell` in
# dimension :math:`d_x = 2`.

# %%
import openturns as ot
import openturns.viewer as otv
from matplotlib import pyplot as plt

# %%
# In the first example, we print the indices involved in
# Smolyak-Legendre quadrature of level 3.
# The multi-indices are computed using the
# :py:meth:`openturns.SmolyakExperiment.computeCombination` method.
# Actually, the multi-indices do not actually depend on the
# underlying univariate quadratures, but this is required for
# the :class:`openturns.SmolyakExperiment` class.

collection = [ot.GaussProductExperiment()] * 2
level = 3
print("level = ", level)
experiment = ot.SmolyakExperiment(collection, level)
indices = experiment.computeCombination()
print(indices)

# %%
# We see that the multi-indices have a sum which is equal to either 3 or 4.
# In other words, these multi-indices belong to two different layers of
# constant 1-norms.
#
# In order to see how this evolves depending on the level of the quadrature,
# the following function creates a 2D plot of the set of multi-indices.


def drawSmolyakIndices(level):
    # Plot Smolyak indices of given level in 2 dimensions
    collection = [ot.GaussProductExperiment()] * 2
    experiment = ot.SmolyakExperiment(collection, level)
    indices = experiment.computeCombination()
    sample = indices
    graph = ot.Graph("L = %d" % (level), "k1", "k2")
    cloud = ot.Cloud(sample)
    cloud.setPointStyle("bullet")
    graph.add(cloud)
    return graph


# %%
# In the following script, we create a grid of plots, where
# each graph corresponds to a given quadrature level.
# The bounding box of each graph is set to a constant value, so that all graphs
# have the same X and Y bounds.

levelMax = 8.0
boundingBox = ot.Interval([0.0] * 2, [levelMax] * 2)
nbrows = 2
nbcols = 3
grid = ot.GridLayout(nbrows, nbcols)
level = 1
for i in range(nbrows):
    for j in range(nbcols):
        graph = drawSmolyakIndices(level)
        if i < nbrows - 1:
            graph.setXTitle("")
        if j > 0:
            graph.setYTitle("")
        graph.setBoundingBox(boundingBox)
        grid.setGraph(i, j, graph)
        level += 1
view = otv.View(grid, figure_kw={"figsize": (5.0, 3.0)})
plt.subplots_adjust(wspace=0.5, hspace=0.7)
plt.tight_layout()

# %%
# We see that when the level increases, the set of Smolyak multi-indices
# correspond to two different layers of constant 1-norm.
# This is a consequence of Smolyak's quadrature, which is based on
# tensorization of univariate difference quadratures.

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# size of timeGrid
size = 6
dimension = 1
sample = ot.Sample(size, dimension)
for i in range(size):
    for j in range(dimension):
        sample[i, j] = i + j + 1

# TimeGrid
timeGrid = ot.RegularGrid(0.0, 1.0 / (size - 1), size)

# TimeSeries
timeSeries = ot.TimeSeries(timeGrid, sample)

# We create an empty ProcessSample with default constructor
psample0 = ot.ProcessSample()
psample0.setName("PSample0")
print("Default constructor")
print("psample0=", psample0)

# We create an empty ProcessSample with timeGrid, size and dimension
# arguments
psample1 = ot.ProcessSample(timeGrid, 4, dimension)
psample1.setName("PSample1")
print("Constructor based on size, dimension and timeGrid")
print("psample1=", psample1)

# change the first component using operator []
psample1[0] = timeSeries
print("changing psample1[0] with []")
print("psample1[0]=", (psample1[0]))

# We create a ProcessSample with size and timeSeries arguments
psample2 = ot.ProcessSample(3, timeSeries)
psample2.setName("PSample2")
print("Constructor based on size / timeSeries")
print("psample2=", psample2)

psample3 = ot.ProcessSample(8, timeSeries)
psample3.erase(2)
assert psample3.getSize() == 7, "wrong size"
psample3.erase(2, 5)
assert psample3.getSize() == 4, "wrong size"
psample3.clear()
assert psample3.getSize() == 0, "wrong size"

# TimeSeriesCollection
# collection = TimeSeriesCollection()
# collection.add(timeSeries)

# sample *= [0.1]
# newtimeSeries = TimeSeries(timeGrid, sample)
# collection.add(newtimeSeries)

# We create a ProcessSample with collection arguments arguments
# psample3 = ProcessSample(collection)

# psample3.setName("PSample3")
# print "Constructor based on collection "
# print "psample3=", psample3

# ts = psample3[1]
# print "last element of psample3=", (ts)
# print "psample3 mean=", psample3.computeMean()
# print "psample3 temporal mean=", psample3.computeTemporalMean()

# A process with 1D domain dimension and 2D output dimension
numberOfIntervals = 20
interval = ot.Interval(-1.0, 1.0)
mesh = ot.IntervalMesher([numberOfIntervals - 1]).build(interval)
outputDimension = 2
sampleSize = 250

processSample = ot.ProcessSample(mesh, sampleSize, outputDimension)
# processSample.setDescription(['Twater', 'Tair'])

R = ot.CorrelationMatrix(outputDimension)
R[0, 1] = 0.75
distribution = ot.Normal([15, 20], [5, 10], R)
for i in range(sampleSize):
    processSample[i] = distribution.getSample(mesh.getVerticesNumber())
mean = processSample.computeMean()
stddev = processSample.computeStandardDeviation()
sample_node4 = processSample.getSampleAtVertex(4)
assert len(sample_node4) == len(processSample), "wrong len"
print("mean=", mean.getValues())
print("sttdev=", stddev.getValues())
graph1 = processSample.draw()
graph2 = processSample.drawMarginalCorrelation(0, 1)
graph3 = processSample.drawCorrelation()
# center
processSample -= processSample.computeMean().getValues()
if 0:
    from openturns.viewer import View

    View(graph1).save("graph1.png")
    View(graph2).save("graph2.png")
    View(graph3).save("graph3.png")

# ctor from collection of Samples
coll = [
    ot.Normal(outputDimension).getSample(mesh.getVerticesNumber())
    for i in range(sampleSize)
]
processSample2 = ot.ProcessSample(mesh, coll)
assert len(processSample2) == sampleSize, "wrong size"

# More statistical methods
# processSample = ot.GaussianProcess(ot.MaternModel([10.0], [0.1], 1.5),  ot.RegularGrid(0.0, 0.1, 1000)).getSample(10000)
print("min=", processSample.getMin().getValues())
print("max=", processSample.getMax().getValues())
print("range=", processSample.computeRange().getValues())
print("variance=", processSample.computeVariance().getValues())
print("skewness=", processSample.computeSkewness().getValues())
print("kurtosis=", processSample.computeKurtosis().getValues())
print("centered moment (3)=", processSample.computeCentralMoment(3).getValues())
print("raw moment (3)=", processSample.computeRawMoment(3).getValues())
x = [0.2] * processSample.getDimension()
print("median=", processSample.computeMedian().getValues())
q = 0.3
print(
    "quantile at level",
    q,
    "=",
    processSample.computeQuantilePerComponent(q).getValues(),
)
print("empirical CDF at", x, "=", processSample.computeEmpiricalCDF(x).getValues())
print(
    "complementary empirical CDF at",
    x,
    "=",
    processSample.computeEmpiricalCDF(x, True).getValues(),
)

n = mesh.getVerticesNumber()
try:
    s = processSample.getSampleAtVertex(n)
    assert False, "should not go there"
except Exception:
    pass



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# Constructor with size
point1 = ot.PointWithDescription(2)
point1[0] = 100.0
point1[1] = 101.0

# Copy constructor
point2 = ot.PointWithDescription(point1)

# Description
print("point1.getDescription() = ", repr(point1.getDescription()))
print("point2.getDescription() = ", repr(point2.getDescription()))

description = ot.Description(2)
description[0] = "point1_X"
description[1] = "point1_Y"

point1.setDescription(description)

print("point1.getDescription() = ", repr(point1.getDescription()))
print("point2.getDescription() = ", repr(point2.getDescription()))



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# for fft, the best implementation is given for N = 2^p
size = 16

# collection for test
collection = ot.ComplexCollection(size)

# Fill the data with artificial values

# Create a complex gaussian sample
for index in range(size):
    realPart = 0.1 * (index + 1.0) / size
    imagPart = 0.3 * (index + 1.0) / size
    collection[index] = realPart + 1j * imagPart

myFFT = ot.KissFFT()
print("myFFT = ", myFFT)

# Initial transformation
print("collection = ", collection)

# FFT transform
transformedCollection = ot.ComplexCollection(myFFT.transform(collection))
print("FFT result = ", transformedCollection)

# Inverse transformation
inverseTransformedCollection = ot.ComplexCollection(
    myFFT.inverseTransform(transformedCollection)
)
print("FFT back=", inverseTransformedCollection)

# 2D case now
N = 8
distribution = ot.Normal(N)
sample = distribution.getSample(2 * N)

# FFT transform
transformedSample = myFFT.transform2D(sample)
print("2D FFT result = ", repr(transformedSample))

# Inverse transformation
inverseTransformedSample = myFFT.inverseTransform2D(transformedSample)
print("2D FFT back=", repr(inverseTransformedSample.real()))

# 3D case
elements = [ot.RandomGenerator.Generate() for i in range(N * N * N)]
tensor = ot.ComplexTensor(N, N, N, elements)

# FFT transform
transformedTensor = myFFT.transform3D(tensor)
print("3D FFT result = ", repr(transformedTensor))

# Inverse transformation
inverseTransformedTensor = myFFT.inverseTransform3D(transformedTensor)
print("3D FFT back=", repr(inverseTransformedTensor.real()))



"""
Extract the copula from a distribution
======================================
"""

# %%
# In this example we are going to retrieve the copula from a multidimensional distribution.
#

# %%
import openturns as ot


# %%
# Create a multivariate Normal distribution
mu = [0.0] * 2
sigma = [1.0] * 2
R = ot.CorrelationMatrix(2)
R[0, 1] = 0.25
distribution = ot.Normal(mu, sigma, R)
print(distribution)

# %%
# Extract the copula
copula = distribution.getCopula()
print(copula)



"""
Use case : axial stressed beam
==============================
"""

import openturns as ot


class AxialStressedBeam:
    """
    Data class for the axial stressed beam example.


    Attributes
    ----------

    dim : The dimension of the problem
          dim=2.

    D : Constant
        Diameter D = 0.02 (m)

    model : `SymbolicFunction`
            The limit state function.

    muR : Constant
          muR=3.0e6, yield strength mean

    sigmaR : Constant
             sigmaR = 3.0e5, yield strength variance

    distribution_R : `LogNormalMuSigma` distribution of the yield strength
                      ot.LogNormalMuSigma(muR, sigmaR, 0.0).getDistribution()

    muF : Constant
          muF=750.0, traction load mean

    sigmaF : Constant
             sigmaR = 50.0, traction load variance

    distribution_F : `Normal` distribution of the traction load
                     ot.Normal(muF, sigmaF)

    distribution : `JointDistribution`
                   The joint distribution of the input parameters.

    Examples
    --------
    >>> from openturns.usecases import stressed_beam
    >>> # Load the axial stressed beam
    >>> sm = stressed_beam.AxialStressedBeam()
    """

    def __init__(self):
        self.dim = 2
        self.D = 0.02
        # Random variable : R
        self.muR = 3.0e6
        self.sigmaR = 3.0e5
        # Random variable : F
        self.muF = 750.0
        self.sigmaF = 50.0
        # create the limit state function model
        self.model = ot.SymbolicFunction(["R", "F"], ["R-F/(pi_/10000.0)"])

        # Yield strength
        self.distribution_R = ot.LogNormalMuSigma(
            self.muR, self.sigmaR, 0.0
        ).getDistribution()
        self.distribution_R.setName("Yield strength")
        self.distribution_R.setDescription("R")
        # Traction load
        self.distribution_F = ot.Normal(self.muF, self.sigmaF)
        self.distribution_F.setName("Traction_load")
        self.distribution_F.setDescription("F")

        # Joint distribution of the input parameters
        self.distribution = ot.JointDistribution(
            [self.distribution_R, self.distribution_F]
        )



"""
Estimate a spectral density function
====================================
"""

# %%
# The objective of this example is to estimate the spectral density
# function :math:`S` from data, which can be a sample of time series or one time
# series.
#
# The following example illustrates the case where the available data is a
# sample of :math:`10^3` realizations of the process, defined on the time grid
# :math:`[0, 102.3]`, discretized every :math:`\Delta t = 0.1`. The spectral model of
# the process is the Cauchy model parameterized by :math:`\vect{\lambda}=(5)`
# and :math:`\vect{a}=(3)`.
#
# The figure draws the graph of the real spectral
# model and its estimation from the sample of time series.
#

# %%
import openturns as ot
import openturns.viewer as otv


# %%
# Generate some data

# Create the time grid
# In the context of the spectral estimate or Fourier transform use,
# we use data blocs with size of form 2^p
tMin = 0.0
tstep = 0.1
size = 2**12
tgrid = ot.RegularGrid(tMin, tstep, size)

# We fix the parameter of the Cauchy model
amplitude = [5.0]
scale = [3.0]
model = ot.CauchyModel(amplitude, scale)
process = ot.SpectralGaussianProcess(model, tgrid)

# Get a time series or a sample of time series
tseries = process.getRealization()
sample = process.getSample(1000)

# %%
# Build a spectral model factory
segmentNumber = 10
overlapSize = 0.3
factory = ot.WelchFactory(ot.Hann(), segmentNumber, overlapSize)

# %%
# Estimation on a TimeSeries or on a ProcessSample
estimatedModel_TS = factory.build(tseries)
estimatedModel_PS = factory.build(sample)

# %%
# Change the filtering window
factory.setFilteringWindows(ot.Hamming())

# %%
# Get the frequencyGrid
frequencyGrid = ot.SpectralGaussianProcess(estimatedModel_PS, tgrid).getFrequencyGrid()

# %%

# With the model, we want to compare values
# We compare values computed with theoritical values
plotSample = ot.Sample(frequencyGrid.getN(), 3)

# Loop of comparison ==> data are saved in plotSample
for k in range(frequencyGrid.getN()):
    freq = frequencyGrid.getStart() + k * frequencyGrid.getStep()
    plotSample[k, 0] = freq
    plotSample[k, 1] = abs(estimatedModel_PS(freq)[0, 0])
    plotSample[k, 2] = abs(model(freq)[0, 0])

# Some cosmetics : labels, legend position, ...
graph = ot.Graph(
    "Estimated spectral function - Validation",
    "Frequency",
    "Spectral density function",
    True,
    "upper right",
)
graph.setLogScale(ot.GraphImplementation.LOGY)

# The first curve is the estimate density as function of frequency
curve1 = ot.Curve(plotSample.getMarginal([0, 1]))
curve1.setColor("blue")
curve1.setLegend("estimate model")

# The second curve is the theoritical density as function of frequency
curve2 = ot.Curve(plotSample.getMarginal([0, 2]))
curve2.setColor("red")
curve2.setLegend("Cauchy model")

graph.add(curve1)
graph.add(curve2)
view = otv.View(graph)
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

continuousDistributionCollection = ot.DistributionCollection()
discreteDistributionCollection = ot.DistributionCollection()
distributionCollection = ot.DistributionCollection()

beta = ot.Beta(2.0, 1.0, 0.0, 1.0)
distributionCollection.add(beta)
continuousDistributionCollection.add(beta)

gamma = ot.Gamma(1.0, 2.0, 3.0)
distributionCollection.add(gamma)
continuousDistributionCollection.add(gamma)

gumbel = ot.Gumbel(1.0, 2.0)
distributionCollection.add(gumbel)
continuousDistributionCollection.add(gumbel)

lognormal = ot.LogNormal(1.0, 1.0, 2.0)
distributionCollection.add(lognormal)
continuousDistributionCollection.add(lognormal)

logistic = ot.Logistic(1.0, 1.0)
distributionCollection.add(logistic)
continuousDistributionCollection.add(logistic)

normal = ot.Normal(1.0, 2.0)
distributionCollection.add(normal)
continuousDistributionCollection.add(normal)

truncatednormal = ot.TruncatedNormal(1.0, 1.0, 0.0, 3.0)
distributionCollection.add(truncatednormal)
continuousDistributionCollection.add(truncatednormal)

student = ot.Student(10.0, 10.0, 1.0)
distributionCollection.add(student)
continuousDistributionCollection.add(student)

triangular = ot.Triangular(-1.0, 2.0, 4.0)
distributionCollection.add(triangular)
continuousDistributionCollection.add(triangular)

uniform = ot.Uniform(1.0, 2.0)
distributionCollection.add(uniform)
continuousDistributionCollection.add(uniform)

weibull = ot.WeibullMin(1.0, 1.0, 2.0)
distributionCollection.add(weibull)
continuousDistributionCollection.add(weibull)

geometric = ot.Geometric(0.5)
distributionCollection.add(geometric)
discreteDistributionCollection.add(geometric)

poisson = ot.Poisson(2.0)
distributionCollection.add(poisson)
discreteDistributionCollection.add(poisson)

x = [[1.0], [2.0], [3.0]]
p = [0.3, 0.2, 0.5]
userdefined = ot.FiniteDiscreteDistribution(x, p)
distributionCollection.add(userdefined)
discreteDistributionCollection.add(userdefined)

size = 10000

# Number of continuous distributions
continuousDistributionNumber = continuousDistributionCollection.getSize()
# Number of discrete distributions
discreteDistributionNumber = discreteDistributionCollection.getSize()
# Number of distributions
distributionNumber = continuousDistributionNumber + discreteDistributionNumber

# We create a collection of Sample of size "size" and of
# dimension 1 (scalar values) : the collection has distributionNumber
# Samples
sampleCollection = [ot.Sample(size, 1) for i in range(distributionNumber)]
# We create a collection of Sample of size "size" and of
# dimension 1 (scalar values) : the collection has
# continuousDistributionNumber Samples
continuousSampleCollection = [
    ot.Sample(size, 1) for i in range(continuousDistributionNumber)
]
# We create a collection of Sample of size "size" and of
# dimension 1 (scalar values) : the collection has
# discreteDistributionNumber Samples
discreteSampleCollection = [
    ot.Sample(size, 1) for i in range(discreteDistributionNumber)
]

for i in range(continuousDistributionNumber):
    continuousSampleCollection[i] = continuousDistributionCollection[i].getSample(size)
    continuousSampleCollection[i].setName(continuousDistributionCollection[i].getName())
    sampleCollection[i] = continuousSampleCollection[i]
for i in range(discreteDistributionNumber):
    discreteSampleCollection[i] = discreteDistributionCollection[i].getSample(size)
    discreteSampleCollection[i].setName(discreteDistributionCollection[i].getName())
    sampleCollection[continuousDistributionNumber + i] = discreteSampleCollection[i]

# Test the normality of several samples using the Anderson Darling test
andersonDarlingResult = ot.Point(distributionNumber)
for i in range(distributionNumber):
    result = ot.NormalityTest.AndersonDarlingNormal(sampleCollection[i])
    andersonDarlingResult[i] = result.getBinaryQualityMeasure()
    print(
        "sample ", sampleCollection[i].getName(), " result=", andersonDarlingResult[i]
    )
print("andersonDarlingResult=", repr(andersonDarlingResult))

# Test the normality of several samples using the Cramer Von Mises test
cramerVonMisesResult = ot.Point(distributionNumber)
for i in range(distributionNumber):
    result = ot.NormalityTest.CramerVonMisesNormal(sampleCollection[i])
    cramerVonMisesResult[i] = result.getBinaryQualityMeasure()
    print("sample ", sampleCollection[i].getName(), " result=", cramerVonMisesResult[i])
print("cramerVonMisesResult=", repr(cramerVonMisesResult))



"""
Compare the distribution of quantile estimators
===============================================
"""

# %%
# In this example, we consider the quantile of level :math:`\alpha` of a distribution
# :math:`X`. The objective is to estimate some confidence intervals based on order statistics
# that bound the quantile
# :math:`x_{\alpha}` with a given confidence :math:`\beta \in [0,1]`.
#
# We define three quantile estimators which are all defined as particular order statistics:
#
# - the empirical estimator of :math:`x_{\alpha}`,
# - the estimator of an upper bound of :math:`x_{\alpha}` with confidence :math:`\beta`,
# - the estimator of an lower bound of :math:`x_{\alpha}` with confidence :math:`\beta`.
#
# We draw the distribution of each estimator.
#
# In this example, we consider the quantile of level :math:`\alpha = 95\%`,
# with a confidence level :math:`\beta = 90\%`.
#
# See  :ref:`quantile_confidence_estimation` and :ref:`quantile_asymptotic_confidence_estimation` to get theoretical details.
import openturns as ot
import openturns.experimental as otexp
import openturns.viewer as otv


# %%
# We consider a scalar random variable :math:`X`. Its distribution has no importance but is assumed to have
# a density.
X_dist = ot.Normal(0, 1)

# %%
# We define the level :math:`\alpha` of the quantile and the confidence level :math:`\beta`.
alpha = 0.95
beta = 0.90

# %%
# The exact value of :math:`x_{\alpha}` is known. We compute it in order to compare the
# estimators to the exact value.
x_alpha_exact = X_dist.computeQuantile(alpha)[0]
print("Exact quantile = ", x_alpha_exact)

# %%
# We generate a sample of the variable.
n = 501
X_sample = X_dist.getSample(n)

# %%
# Now, we assume to know :math:`X` only through the generated sample.
#
# The empirical estimator of :math:`x_{\alpha}` is defined by the
# :math:`k_{emp}` -th order statistics, denoted
# by :math:`X_{(k_{emp})}`,
# where :math:`k_{emp} = (\lceil \sampleSize \alpha \rceil) -1`. The numerotation
# :math:`\lceil x \rceil` designs the smallest integer value that is greater
# than or equal to :math:`x` (we subtract 1 because python numbering starts at 0).
# The value of this order statistics is computed on the sample.
k_emp = int(n * alpha)
empiricalQuantile = X_sample.computeQuantile(alpha)
print("Empirical quantile = ", empiricalQuantile)
print("Empirical order statistics = ", k_emp)

# %%
# Now, we want to get the exact unilateral confidence interval that provides an upper bound of :math:`x_{\alpha}` with the confidence
# :math:`\beta`, which means that :math:`\Prob{X_{(k_{up})} \geq x_{\alpha}} \geq \beta`.
# This can be done using :class:`~openturns.experimental.QuantileConfidence`.
# We can get the order :math:`k_{up}` of the useful statistics. Care that the enumeration begins at 0.
quantConf = otexp.QuantileConfidence(alpha, beta)
k_up = quantConf.computeUnilateralRank(n)

# %%
# We can directly get the order statistics :math:`X_{(k_{up})}` evaluated on the sample.
upper_bound = quantConf.computeUnilateralConfidenceInterval(X_sample)
print("Upper bound of the quantile = ", upper_bound)
print("Upper bound order statistics = ", k_up)

# %%
# To get the exact unilateral confidence interval that provides a lower bound of :math:`x_{\alpha}` with the confidence
# :math:`\beta`, which means that :math:`\Prob{X_{(k_{low})} \leq x_{\alpha}} \geq \beta`.
# This can also be done using :class:`~openturns.experimental.QuantileConfidence`.
# We can get the order :math:`k_{low}` of the useful statistics. Care: the enumeration begins at 0 here!
k_low = quantConf.computeUnilateralRank(n, True)

# %%
# We can directly get the order statistics :math:`X_{(k_{low})}` evaluated on the sample.
lower_bound = quantConf.computeUnilateralConfidenceInterval(X_sample, True)
print("Lower bound of the quantile = ", lower_bound)
print("Lower bound order statistics = ", k_low)

# %%
# In order to draw the distribution of any order statistics of :math:`X`, we first create the distribution
# of the :math:`\sampleSize` order statistics of :math:`X`, denoted by :math:`\vect{X}_{(0:\sampleSize-1)} = (X_{(0)},\dots,X_{(\sampleSize-1)})`.
# This distribution as the following cumulative distribution function:
#
# .. math::
#
#    F_{\vect{X}_{(0:\sampleSize-1)}}(\vect{x}) = F_{\vect{U}_{(0:\sampleSize-1)}}(F(x_1), \dots, F(x_{\sampleSize}-1))
#
#
# and its PDF (if defined) by:
#
# .. math::
#
#    f_{\vect{X}_{(0:\sampleSize-1)}}(\vect{x}) = \sampleSize!\prod_{i=0}^{\sampleSize-1} f(x_i) \,\mathbf{1}_{\cS}(\vect{x})
#
# where :math:`F` and :math:`f` are respectively the cumulative distribution function and
# probability density function of :math:`X` and :math:`\cS \subset \Rset^\sampleSize` is defined by:
#
# .. math::
#
#   \cS=\left\{(x_0, \dots, x_{\sampleSize-1}) \in [0,1]^\sampleSize\,|\,0 \leq x_0 \leq \dots \leq x_{\sampleSize-1} \leq 1 \right\}.
#
# Thus :math:`\vect{X}_{(0:\sampleSize-1)}` is defined as the joint distribution which marginals are all distributed
# as :math:`X` and which core is the order statistics distribution of the Uniform(0,1) distribution.
# The  order statistics distribution of the Uniform(0,1) is created using the class
# :class:`~openturns.UniformOrderStatistics` which only requires to define the dimension of the
# order statistics distribution.
unif_orderStat = ot.UniformOrderStatistics(n)
normal_orderStat = ot.JointDistribution([X_dist] * n, unif_orderStat)

# %%
# Thus, the order statistics :math:`X_{(k_{emp})}`, :math:`X_{(k_{low})}` and :math:`X_{(k_{up})}` are the marginals
# :math:`k_{emp}`, :math:`k_{low}` and :math:`k_{up}` of :math:`\vect{X}_{(0:\sampleSize-1)}`.
X_emp = normal_orderStat.getMarginal(k_emp)
X_up = normal_orderStat.getMarginal(k_up)
X_low = normal_orderStat.getMarginal(k_low)

# %%
# Now we can draw the pdf.
xMin = x_alpha_exact - 0.5
xMax = x_alpha_exact + 1
nPoints = 1001
g = X_emp.drawPDF(xMin, xMax, nPoints)
g.add(X_up.drawPDF(xMin, xMax, nPoints))
g.add(X_low.drawPDF(xMin, xMax, nPoints))

# %%
# We add the exact quantile.
line_exactQuant = ot.Curve(
    [[x_alpha_exact, 0.0], [x_alpha_exact, X_emp.computePDF([x_alpha_exact])]]
)
line_exactQuant.setLineStyle("dashed")
line_exactQuant.setLineWidth(2)
line_exactQuant.setColor("red")
g.add(line_exactQuant)

# %%
# Before viewing the graph, we want to fill the area that corresponds to the events:
#
# .. math::
#
#     \left \{ X_{(k_{up})} \leq x_{\alpha} \right \} \\
#     \left \{ X_{(k_{low})} \geq x_{\alpha} \right \}
#
# Each event has the probability :math:`1- \beta`.
#
# To do that, we define a function that returns a sample created from a regular grid from xmin to xmax with npoints points.


def linearSample(xmin, xmax, npoints):
    step = (xmax - xmin) / (npoints - 1)
    rg = ot.RegularGrid(xmin, step, npoints)
    vertices = rg.getVertices()
    return vertices


# %%
# Then we use it to define some :class:`~openturns.Polygon` that will be filled. These polygons
# define the area under the pdf line. Thus, the lower vertical bound is 0 and the upper vertical bound is the PDF line.
#
# First the area for the upper bound.
nplot = 100
x_list = linearSample(xMin, x_alpha_exact, nplot)
y_list = X_up.computePDF(x_list)
vLow = [0.0] * nplot
vUp = [y_list[i, 0] for i in range(nplot)]
boundsPoly_upper = ot.Polygon.FillBetween(x_list.asPoint(), vLow, vUp)
boundsPoly_upper.setColor(g.getColors()[1])
g.add(boundsPoly_upper)


# %%
# Then the area for the lower bound.
nplot = 100
x_list = linearSample(x_alpha_exact, xMax, nplot)
y_list = X_low.computePDF(x_list)
vLow = [0.0] * nplot
vUp = [y_list[i, 0] for i in range(nplot)]
boundsPoly_lower = ot.Polygon.FillBetween(x_list.asPoint(), vLow, vUp)
boundsPoly_lower.setColor(g.getColors()[2])
g.add(boundsPoly_lower)

# %%
g.setLegends(
    [
        r"$X_{emp}$",
        r"$X_{up} (\beta = $" + str(beta) + ")",
        r"$X_{low} (\beta = $" + str(beta) + ")",
        "exact quantile",
    ]
)
g.setLegendPosition("upper right")

# %%
g.setTitle("Quantile estimation and confidence intervals")
g.setXTitle("x")
view = otv.View(g)

# %%
# Display all the figures.
otv.View.ShowAll()


# %%
# Now we can conclude that:
#
#  * the empirical estimator is centered on the true quantile,
#  * the :math:`X_{(k_{up})}` estimator has a probability :math:`\beta` to be above the true quantile,
#  * the :math:`X_{(k_{low})}` estimator has a probability :math:`\beta` to be under the true quantile.
#



"""
Gaussian Process Regression: propagate uncertainties
====================================================

In this example we propagate uncertainties through a GP metamodel of the :ref:`Ishigami model<use-case-ishigami>`.
"""

import openturns as ot
import openturns.viewer as otv


# %%
# We first build the metamodel and then compute its mean with a Monte-Carlo
# computation.
#
# We load the Ishigami model from the usecases module:

from openturns.usecases import ishigami_function

im = ishigami_function.IshigamiModel()

# %%
# We build a design of experiments with a Latin Hypercube Sampling (LHS) for the three input variables
# supposed independent.
experiment = ot.LHSExperiment(im.distribution, 30, False, True)
xdata = experiment.generate()

# %%
# We get the exact model and evaluate it at the input training data `xdata` to
# build the output data `ydata`.
model = im.model
ydata = model(xdata)

# %%
# We define our GP process:
#
#  - a constant basis in :math:`\mathbb{R}^3` ;
#  - a squared exponential covariance function.
#
dimension = 3
basis = ot.ConstantBasisFactory(dimension).build()
covarianceModel = ot.SquaredExponential([0.1] * dimension, [1.0])
fitter = ot.GaussianProcessFitter(xdata, ydata, covarianceModel, basis)
fitter.run()
fitter_result = fitter.getResult()
algo = ot.GaussianProcessRegression(fitter_result)
algo.run()
result = algo.getResult()

# %%
# We finally get the metamodel to use with Monte-Carlo.
metamodel = result.getMetaModel()

# %%
# We want to estmate the mean of the Ishigami model with Monte-Carlo using the
# metamodel instead of the exact model.
#
# We first create a random vector following the input distribution :
X = ot.RandomVector(im.distribution)

# %%
# And then we create a random vector from the image of the input random vector
# by the metamodel :
Y = ot.CompositeRandomVector(metamodel, X)

# %%
# We now set our :class:`~openturns.ExpectationSimulationAlgorithm` object :
algo = ot.ExpectationSimulationAlgorithm(Y)
algo.setMaximumOuterSampling(50000)
algo.setBlockSize(1)
algo.setCoefficientOfVariationCriterionType("NONE")

# %%
# We run it and store the results :
algo.run()
result = algo.getResult()

# %%
# The expectation ( :math:`\mathbb{E}(Y)` mean ) is obtained with :
expectation = result.getExpectationEstimate()

# %%
# The mean estimate of the metamodel is
print("Mean of the Ishigami metamodel : %.3e" % expectation[0])

# %%
# We draw the convergence history.
graph = algo.drawExpectationConvergence()
view = otv.View(graph)

# %%
# For reference, the exact mean of the Ishigami model is :
print("Mean of the Ishigami model : %.3e" % im.expectation)

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import math as m

ot.TESTPREAMBLE()


# Problem parameters
dimension = 3
a = 7.0
b = 0.1
# Reference analytical values
meanTh = a / 2
covTh = (b**2 * m.pi**8) / 18.0 + (b * m.pi**4) / 5.0 + (a**2) / 8.0 + 1.0 / 2.0
sob_1 = [
    (b * m.pi**4 / 5.0 + b**2 * m.pi**8 / 50.0 + 1.0 / 2.0) / covTh,
    (a**2 / 8.0) / covTh,
    0.0,
]
sob_2 = [0.0, (b**2 * m.pi**8 / 18.0 - b**2 * m.pi**8 / 50.0) / covTh, 0.0]
sob_3 = [0.0]
sob_T1 = [
    sob_1[0] + sob_2[0] + sob_2[1] + sob_3[0],
    sob_1[1] + sob_2[0] + sob_2[2] + sob_3[0],
    sob_1[2] + sob_2[1] + sob_2[2] + sob_3[0],
]
sob_T2 = [
    sob_2[0] + sob_2[1] + sob_3[0],
    sob_2[0] + sob_2[2] + sob_3[0],
    sob_2[1] + sob_2[2] + sob_3[0],
]
sob_T3 = [sob_3[0]]
# Create the Ishigami function
inputVariables = ["xi1", "xi2", "xi3"]
formula = ot.Description(1)
formula[0] = (
    "sin(xi1) + (" + str(a) + ") * (sin(xi2)) ^ 2 + (" + str(b) + ") * xi3^4 * sin(xi1)"
)
model = ot.SymbolicFunction(inputVariables, formula)

# Create the input distribution
distribution = ot.JointDistribution([ot.Uniform(-m.pi, m.pi)] * dimension)

# Create the orthogonal basis
enumerateFunction = ot.LinearEnumerateFunction(dimension)
productBasis = ot.OrthogonalProductPolynomialFactory(
    [ot.LegendreFactory()] * dimension, enumerateFunction
)

# Create the projection strategy
samplingSize = 250
listProjectionStrategy = list()
# Monte Carlo sampling
inputSample = ot.LowDiscrepancyExperiment(
    ot.SobolSequence(), distribution, samplingSize
).generate()
outputSample = model(inputSample)
# From here, the model is no more needed
listProjectionStrategy.append(ot.LeastSquaresStrategy())
listProjectionStrategy.append(
    ot.LeastSquaresStrategy(
        ot.LeastSquaresMetaModelSelectionFactory(ot.LARS(), ot.CorrectedLeaveOneOut())
    )
)
listProjectionStrategy.append(ot.IntegrationStrategy())
# Create the adaptive strategy
# We can choose amongst several strategies
# First, the most efficient (but more complex!) strategy
degree = 6
listAdaptiveStrategy = list()
indexMax = enumerateFunction.getBasisSizeFromTotalDegree(degree)
basisDimension = enumerateFunction.getBasisSizeFromTotalDegree(degree // 2)
threshold = 1.0e-6
listAdaptiveStrategy.append(
    ot.CleaningStrategy(productBasis, indexMax, basisDimension, threshold)
)
# Second, the most used (and most basic!) strategy
listAdaptiveStrategy.append(
    ot.FixedStrategy(
        productBasis, enumerateFunction.getBasisSizeFromTotalDegree(degree)
    )
)

# Check LeastSquaresStrategy
projectionStrategy = ot.LeastSquaresStrategy()
assert projectionStrategy.isLeastSquares()
assert not projectionStrategy.involvesModelSelection()

# Check LeastSquaresMetaModelSelectionFactory
projectionStrategy = ot.LeastSquaresStrategy(
    ot.LeastSquaresMetaModelSelectionFactory(ot.LARS(), ot.CorrectedLeaveOneOut())
)
assert projectionStrategy.isLeastSquares()
assert projectionStrategy.involvesModelSelection()

# Check IntegrationStrategy
projectionStrategy = ot.IntegrationStrategy()
assert not projectionStrategy.isLeastSquares()
assert not projectionStrategy.involvesModelSelection()

# Check CleaningStrategy
adaptiveStrategy = ot.CleaningStrategy(
    productBasis, indexMax, basisDimension, threshold
)
assert adaptiveStrategy.involvesModelSelection()

# Check FixedStrategy
adaptiveStrategy = ot.FixedStrategy(
    productBasis, enumerateFunction.getBasisSizeFromTotalDegree(degree)
)
assert not adaptiveStrategy.involvesModelSelection()

for adaptiveStrategyIndex in range(len(listAdaptiveStrategy)):
    adaptiveStrategy = listAdaptiveStrategy[adaptiveStrategyIndex]
    for projectionStrategyIndex in range(len(listProjectionStrategy)):
        projectionStrategy = listProjectionStrategy[projectionStrategyIndex]
        # Create the polynomial chaos algorithm
        maximumResidual = 1.0e-10
        algo = ot.FunctionalChaosAlgorithm(
            inputSample,
            outputSample,
            distribution,
            adaptiveStrategy,
            projectionStrategy,
        )
        algo.setMaximumResidual(maximumResidual)
        ot.RandomGenerator.SetSeed(0)
        algo.run()

        # Examine the results
        result = algo.getResult()
        print("###################################")
        print(adaptiveStrategy)
        print(algo.getProjectionStrategy())
        print("isLeastSquares= ", result.isLeastSquares())
        isLeastSquaresReference = (
            projectionStrategy.getClassName() == "LeastSquaresStrategy"
        )
        assert result.isLeastSquares() == isLeastSquaresReference
        print("involvesModelSelection= ", result.involvesModelSelection())
        modelSelectionReference = (
            projectionStrategy.involvesModelSelection()
            or adaptiveStrategy.involvesModelSelection()
        )
        assert result.involvesModelSelection() == modelSelectionReference

        # Post-process the results
        vector = ot.FunctionalChaosRandomVector(result)
        mean = vector.getMean()[0]
        print("mean=%.8f" % mean, "absolute error=%.10f" % abs(mean - meanTh))
        variance = vector.getCovariance()[0, 0]
        print(
            "variance=%.8f" % variance, "absolute error=%.10f" % abs(variance - covTh)
        )
        sensitivity = ot.FunctionalChaosSobolIndices(result)
        for i in range(dimension):
            value = sensitivity.getSobolIndex(i)
            print(
                "Sobol index",
                i,
                "= %.8f" % value,
                "absolute error=%.10f" % abs(value - sob_1[i]),
            )
        indices = ot.Indices(2)
        k = 0
        for i in range(dimension):
            indices[0] = i
            for j in range(i + 1, dimension):
                indices[1] = j
                value = sensitivity.getSobolIndex(indices)
                print(
                    "Sobol index",
                    indices,
                    "=%.8f" % value,
                    "absolute error=%.10f" % abs(value - sob_2[k]),
                )
                k = k + 1
        indices = ot.Indices([0, 1, 2])
        value = sensitivity.getSobolIndex(indices)
        print(
            "Sobol index",
            indices,
            "=%.8f" % value,
            "absolute error=%.10f" % abs(value - sob_3[0]),
        )
        for i in range(dimension):
            value = sensitivity.getSobolTotalIndex(i)
            print(
                "Sobol total index",
                i,
                "=%.8f" % value,
                "absolute error=%.10f" % abs(value - sob_T1[i]),
            )
        indices = ot.Indices(2)
        k = 0
        for i in range(dimension):
            indices[0] = i
            for j in range(i + 1, dimension):
                indices[1] = j
                value = sensitivity.getSobolIndex(indices)
                print(
                    "Sobol total index",
                    indices,
                    "=%.8f" % value,
                    "absolute error=%.10f" % abs(value - sob_2[k]),
                )
                k = k + 1
        indices = ot.Indices([0, 1, 2])
        value = sensitivity.getSobolTotalIndex(indices)
        print(
            "Sobol total index ",
            indices,
            "=%.8f" % value,
            "absolute error=%.10f" % abs(value - sob_3[0]),
        )



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


matrix1 = ot.SquareMatrix(2)
matrix1.setName("matrix1")
matrix1[0, 0] = 1.0
matrix1[1, 0] = 2.0
matrix1[0, 1] = 5.0
matrix1[1, 1] = 12.0
print("matrix1 = ", matrix1)

pt = ot.Point()
pt.add(5.0)
pt.add(0.0)
print("pt = ", pt)

result = matrix1.solveLinearSystem(pt)
print("result = ", result)

determinant = matrix1.computeDeterminant()
print("determinant = %.6g" % determinant)

ev = matrix1.computeEigenValues()
print("ev = ", ev)
ev, evect = matrix1.computeEV()
print("ev=", ev)
print("evect=", evect)
print("evect=")
print(evect.__str__())
maxModule = matrix1.computeLargestEigenValueModule(10, 1e-2)
print("max |ev|=%.6g" % maxModule)

# Check the high dimension determinant computation
matrix2 = ot.SquareMatrix(3)
matrix2[0, 0] = 1.0
matrix2[0, 1] = 2.0
matrix2[0, 2] = 3.0
matrix2[1, 0] = -1.5
matrix2[1, 1] = 2.5
matrix2[1, 2] = -3.5
matrix2[2, 0] = 1.5
matrix2[2, 1] = -3.5
matrix2[2, 2] = 2.5

print("matrix2=")
print(matrix2.__str__())
# Need a specific Python wrapping, e.g returning both value and sign
# sign = 0.0
# value = matrix2.computeLogAbsoluteDeterminant(sign)
# print "log(|det|)=", value, ", sign=", sign
determinant = matrix2.computeDeterminant()
print("determinant = %.6g" % determinant)



"""
Create a parametric function
============================
"""

# %%
# In this example, we show how to use the :class:`~openturns.ParametricFunction` class.
# This is a tool which is very convenient when we perform
# calibration, e.g. with :class:`~openturns.NonLinearLeastSquaresCalibration`
# or :class:`~openturns.RandomWalkMetropolisHastings`.

# %%
# In this example we create a parametric function:
#
# .. math::
#    d_{L,I}(E, F):  \Rset^2  \rightarrow \Rset
#
# function from an existing "full" function:
#
# .. math::
#  d(E, F, L, I):  \Rset^4  \rightarrow \Rset.
#

# %%
import openturns as ot


# %%
# Define the function
# ~~~~~~~~~~~~~~~~~~~

# %%
# Create the function with all parameters d(E, F, L, I)


def d_func(X):
    E, F, L, II = X
    d = -F * L**3 / (3.0 * E * II)
    return [d]


beam = ot.PythonFunction(4, 1, d_func)
beam.setInputDescription(["E", "F", "L", "I"])

# %%
# Evaluate d
x = [50.0, 1.0, 10.0, 5.0]
beam(x)

# %%
# In the physical model, the inputs and parameters are ordered as
# presented in the next table.
# Notice that there are no parameters in the physical model.
#
# +-------+----------------+
# | Index | Input variable |
# +=======+================+
# | 0     | E              |
# +-------+----------------+
# | 1     | F              |
# +-------+----------------+
# | 2     | L              |
# +-------+----------------+
# | 3     | I              |
# +-------+----------------+
#
# +-------+-----------+
# | Index | Parameter |
# +=======+===========+
# | ∅     | ∅         |
# +-------+-----------+
#
# **Table 1.** Indices and names of the inputs and parameters of the physical model.
#

# %%
# The next cell presents the description of the input variables
# and the description of the parameters of the physical model.
# We see that there is no parameter at this stage in this function.
print("Physical Model Inputs:", beam.getInputDescription())
print("Physical Model Parameters:", beam.getParameterDescription())


# %%
# Define the parametric function
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# %%
# We create a :class:`~openturns.ParametricFunction`
# where the parameters are `L` and `I` which have the indices 2 and
# and 3 in the physical model.
#
# +-------+----------------+
# | Index | Input variable |
# +=======+================+
# | 0     | E              |
# +-------+----------------+
# | 1     | F              |
# +-------+----------------+
#
# +-------+-----------+
# | Index | Parameter |
# +=======+===========+
# | 0     | L         |
# +-------+-----------+
# | 1     | I         |
# +-------+-----------+
#
# **Table 2.** Indices and names of the inputs and parameters of the parametric model.
#

# %%
# Create the indices of the frozen parameters (L,I) from the full parameter list
indices = [2, 3]

# %%
# Create the values of the frozen parameters (L,I)
referencePoint = [10.0, 5.0]

# %%
# Create the parametric function
beam_LI = ot.ParametricFunction(beam, indices, referencePoint)

# %%
# The next cell presents the description of the input variables
# and the description of the parameters of the parametric function.
# We see that the parametric function has 2 parameters: L and I.
print("Physical Model Inputs:", beam_LI.getInputDescription())
print("Physical Model Parameters:", beam_LI.getParameterDescription())

# %%
# Evaluate d on (E,F) with fixed parameters (L,I)
beam_LI([50.0, 1.0])



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
import math as m

ot.TESTPREAMBLE()

ot.RandomGenerator.SetSeed(1)

# Sample from a target distribution defined through its log-PDF
# (defined up to some additive constant) and its support:

log_density = ot.SymbolicFunction("x", "log(2 + sin(x)^2)")
support = ot.Interval([0.0], [2.0 * m.pi])

# Apply a Metropolis adjusted Langevin algorithm (MALA).

initialState = [2.5]
proposal = ot.Normal()
h = 0.1
std_deviation = m.sqrt(h)


# The mean of the proposal normal distribution is the current state,
# but moved according to the derivative of the target log-density.


def python_link(x):
    derivative_log_density = log_density.getGradient().gradient(x)[0, 0]
    mean = x[0] + h / 2 * derivative_log_density
    return [mean, std_deviation]


link = ot.PythonFunction(1, 2, python_link)
mala = ot.UserDefinedMetropolisHastings(
    log_density, support, initialState, proposal, link
)

# %%
# Get a sample

sampleSize = 100000
sample = mala.getSample(sampleSize)


# %%
# Compute sample mean and median

mean_ref = m.pi
sample_mean = sample.computeMean()[0]
ott.assert_almost_equal(sample_mean, mean_ref, 1e-2, 0.0)

median_ref = m.pi
sample_median = sample.computeQuantile(0.5)[0]
ott.assert_almost_equal(sample_median, median_ref, 1e-1, 0.0)



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv

# Create a bivariate Gaussian process
myTG = ot.RegularGrid(0.0, 0.01, 100)
myCov = ot.ExponentialModel([3.0], [1.2])
myProcess = ot.GaussianProcess(myCov, myTG)

myTS = myProcess.getRealization()

graph = myTS.drawMarginal(0)
graph.setTitle("A time series")

fig = plt.figure(figsize=(8, 4))
axis = fig.add_subplot(111)
axis.set_xlim(auto=True)

otv.View(graph, figure=fig, axes=[axis], add_legend=True)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Analytical construction
input = "x"
formulas = "x-cos(2*x)"
analytical = ot.SymbolicFunction([input], [formulas])
myAlgo = ot.Bisection()
print("myAlgo=", myAlgo)
value = 0.0
print("Solve ", formulas, "=%.5f" % value, " for ", input)
print("x=%.5f" % myAlgo.solve(analytical, value, 0.0, 3.0))
print("myAlgo=", myAlgo)



"""
============================================================================
Create a multivariate basis of functions from scalar multivariable functions
============================================================================
"""

# %%
# Description
# ===========
#
# We want to build a finite basis of functions :math:`\mathcal{B}=\{f_1,...,f_n\}` where each
# :math:`f_i: \mathbb{R}^d \mapsto \mathbb{R}^q` with :math:`d>1, q>1`, from given scalar multivariable
# functions :math:`g_i: \mathbb{R}^d \mapsto \mathbb{R}`.
#
# In other words, we want to create a basis of functions having both a multivariate input *and* output from
# functions having a multivariate input and a scalar output.
#
# In this example, we do that by the aggregation of the given functions :math:`g_i`,
# using the :class:`~openturns.AggregatedFunction` class. This class allows one to create a multivariate output
# function from a list of functions having a univariate output dimension. Note that we can only aggregate functions
# that share the same input space.
#
# We illustrate two cases that differ from the way the functions :math:`g_i` have been created:
#
# - Case 1: we suppose that the functions :math:`g_i` have no particular origin,
# - Case 2: we suppose that the functions :math:`g_i` have been built from particular bases.
#
# In both cases, we use the :class:`~openturns.Basis` class to create the basis and the
# :class:`~openturns.AggregatedFunction` class to create a multivariate output function from a
# list of scalar multivariable functions.
#
# Case 1: Aggregation of given functions
# ======================================
#
# In that case, we have some functions :math:`g_i: \mathbb{R}^d \mapsto \mathbb{R}` and we aggregate them
# in order to build the basis :math:`\mathcal{B}`.
#
# For example, we set :math:`d=2`, :math:`q=3` and we use the functions :math:`(g_1, g_2, g_3)` defined by:
#
# .. math::
#     \begin{array}{lcl}
#       g_1(x_1, x_2) & = & 1\\
#       g_2(x_1, x_2) & = & x_1 + x_2\\
#       g_3(x_1, x_2) & = & x_1^2 + x_2^2
#     \end{array}
#
# and we want to build the finite basis :math:`\mathcal{B}=\{f_1,f_2\}` defined by:
#
# .. math::
#     \begin{array}{lcl}
#       f_1(x_1, x_2) & = & (1, 1, 1)\\
#       f_2(x_1, x_2) & = & (1, x_1 + x_2, x_1^2 + x_2^2)
#     \end{array}
import openturns as ot

d = 2
q = 3

# Create the functions g_i
g_1 = ot.SymbolicFunction(["x1", "x2"], ["1"])
g_2 = ot.SymbolicFunction(["x1", "x2"], ["x1+x2"])
g_3 = ot.SymbolicFunction(["x1", "x2"], ["x1^2+x2^2"])

# Create the multivariate basis
f_1 = ot.AggregatedFunction([g_1] * q)
f_2 = ot.AggregatedFunction([g_1, g_2, g_3])
basis = ot.Basis([f_1, f_2])
print("Fonction f_1 : ", basis.build(0))

# %%
# Case 2: Aggregation of functions built from given bases
# =======================================================
#
# In that case, we use some functions :math:`g_i: \mathbb{R}^d \mapsto \mathbb{R}` that have been
# built from particular bases. For example, they are members of polynomial families which are orthonormal
# to given distributions.
#
# In this example, we use both polynomial families:
#
# - :math:`(g_1^i)_{i\geq 0}` which is a orthonormal multivariate polynomial basis with respect to the uniform
#   distributipn on :math:`[-1,1]^d` (it is obtained through a tensor product of univariate Legendre polynomials
#   orthonormal with respect to the uniform distribution on :math:`[-1,1]`),
# - :math:`(g_2^i)_{i\geq 0}` which is a orthonormal multivariate polynomial basis with respect to the multivariate
#   standard normal distribution (it is obtained through a tensor product of univariate Hermite polynomials
#   orthonormal with respect to the standard normal univariate distribution).
#
# We consider the linear
# enumeration of the polynomial family. The functions :math:`f_i` are built as the aggregation of
# the :math:`i`-th polynomials of the families :math:`(g_1^k)_k` and :math:`(g_2^k)_k`.
# We only consider :math:`g_1^i` and :math:`g_2^i` of total degree less than 2:
# then we have 6 polynomials :math:`f_i`.
#
# We still set :math:`d=2`, :math:`q=3`.
d = 2

legendre_builder = ot.OrthogonalProductPolynomialFactory([ot.LegendreFactory()] * d)
hermite_builder = ot.OrthogonalProductPolynomialFactory([ot.HermiteFactory()] * d)

# Generate the multivariate functions f_i (quadratic)
total_deg = 2
basis_size = ot.LinearEnumerateFunction(d).getStrataCumulatedCardinal(total_deg)
basis = []
for i in range(basis_size):
    f_i = ot.AggregatedFunction([legendre_builder.build(i), hermite_builder.build(i)])
    basis.append(f_i)

basis = ot.Basis(basis)



#!/usr/bin/env python

import openturns as ot
import scipy.stats as st
from numpy import asarray

n1 = 200
n2 = 300


def compare(sample1, sample2):
    print(st.ks_2samp(asarray(sample1).flatten(), asarray(sample2).flatten()))
    print(ot.FittingTest.TwoSamplesKolmogorov(sample1, sample2))


sample1 = ot.Normal().getSample(n1)
sample2 = ot.Normal(0.5, 1.5).getSample(n2)
compare(sample1, sample2)

sample3 = ot.Normal(0.01, 1.0).getSample(n2)
compare(sample1, sample3)

sample4 = ot.Normal().getSample(n2)
compare(sample1, sample4)



"""
Cross Entropy Importance Sampling
=================================
"""

# %%
#
# The objective is to evaluate a failure probability using Cross Entropy Importance Sampling.
# Two versions in the standard or physical spaces are implemented.
# See :class:`~openturns.StandardSpaceCrossEntropyImportanceSampling` and :class:`~openturns.PhysicalSpaceCrossEntropyImportanceSampling`.
# We consider the simple stress beam example: :ref:`axial stressed beam <use-case-stressed-beam>`.


# %%
# First, we import the python modules:

# %%
import openturns as ot
import openturns.viewer as otv
from openturns.usecases import stressed_beam

# %%
# Create the probabilistic model
# ------------------------------

# %%
axialBeam = stressed_beam.AxialStressedBeam()
distribution = axialBeam.distribution
print("Initial distribution:", distribution)

# %%
# Draw the limit state function :math:`g` to help to understand the shape of the limit state.

# %%
g = axialBeam.model
graph = ot.Graph("Simple stress beam", "R", "F")
graph.setLegendPosition("upper right")
drawfunction = g.draw([1.8e6, 600], [4e6, 950.0], [100] * 2)
graph.add(drawfunction)
view = otv.View(graph)

# %%
# Create the output random vector :math:`Y = g(\vect{X})` with :math:`\vect{X} = (R,F)`.

# %%
X = ot.RandomVector(distribution)
Y = ot.CompositeRandomVector(g, X)

# %%
# Create the event :math:`\{ Y = g(\vect{X}) \leq 0 \}`
# -----------------------------------------------------

# %%
threshold = 0.0
event = ot.ThresholdEvent(Y, ot.Less(), 0.0)

# %%
# Evaluate the probability with the Standard Space Cross Entropy
# --------------------------------------------------------------

# %%
# We choose to set the intermediate quantile level to 0.35.

# %%
standardSpaceIS = ot.StandardSpaceCrossEntropyImportanceSampling(event, 0.35)

# %%
# The sample size at each iteration can be changed by the following accessor:

# %%
standardSpaceIS.setMaximumOuterSampling(1000)

# %%
# Now we can run the algorithm and get the results.

# %%
standardSpaceIS.run()
standardSpaceISResult = standardSpaceIS.getResult()
proba = standardSpaceISResult.getProbabilityEstimate()
print("Proba Standard Space Cross Entropy IS = ", proba)
print(
    "Current coefficient of variation = ",
    standardSpaceISResult.getCoefficientOfVariation(),
)

# %%
# The confidence interval of level :math:`95\%` is:
ci95 = standardSpaceISResult.getProbabilityDistribution().computeBilateralConfidenceInterval(
    0.95
)
print("Confidence interval (0.95) = ", ci95)

# %%
# We can analyze the simulation budget.

# %%
print(
    "Numerical budget: ",
    standardSpaceISResult.getBlockSize() * standardSpaceISResult.getOuterSampling(),
)

# %%
# We can also retrieve the optimal auxiliary distribution in the standard space.

# %%
print(
    "Auxiliary distribution in Standard space: ",
    standardSpaceISResult.getAuxiliaryDistribution(),
)


# %%
# Draw initial samples and final samples
# --------------------------------------
#

# %%
# First we get the auxiliary samples in the standard space and we project them in physical space.

# %%
auxiliaryInputSamples = standardSpaceISResult.getAuxiliaryInputSample()
auxiliaryInputSamplesPhysicalSpace = (
    distribution.getInverseIsoProbabilisticTransformation()(auxiliaryInputSamples)
)


# %%
graph = ot.Graph("Cloud of samples and failure domain", "R", "F")
graph.setLegendPosition("upper right")
# Generation of samples with initial distribution
initialSamples = ot.Cloud(
    distribution.getSample(1000), "blue", "plus", "Initial samples"
)
auxiliarySamples = ot.Cloud(
    auxiliaryInputSamplesPhysicalSpace, "orange", "fsquare", "Auxiliary samples"
)
# Plot failure domain
nx, ny = 50, 50
xx = ot.Box([nx], ot.Interval([1.80e6], [4e6])).generate()
yy = ot.Box([ny], ot.Interval([600.0], [950.0])).generate()
inputData = ot.Box([nx, ny], ot.Interval([1.80e6, 600.0], [4e6, 950.0])).generate()
outputData = g(inputData)
mycontour = ot.Contour(xx, yy, outputData)
mycontour.setLevels([0.0])
mycontour.setLabels(["0.0"])
mycontour.setLegend("Failure domain")
graph.add(initialSamples)
graph.add(auxiliarySamples)
graph.add(mycontour)
view = otv.View(graph)

# %%
# In the previous graph, the blue crosses stand for samples drawn with the initial distribution and the orange squares stand for the samples drawn at the final iteration.

# %%
# Estimation of the probability with the Physical Space Cross Entropy
# -------------------------------------------------------------------

# %%
# For a more advanced usage, it is possible to work in the physical space to specify the auxiliary distribution.
# In this second example, we take the auxiliary distribution in the same family as the initial distribution and we want to optimize all the parameters.

# %%
# The parameters to be optimized are the parameters of the native distribution.
# It is necessary to define among all the distribution parameters, which ones will be optimized through the indices of the parameters.
# In this case, all the parameters will be optimized.

# %%
# Be careful that the native parameters of the auxiliary distribution will be considered.
# Here for the :class:`~openturns.LogNormalMuSigma` distribution, this corresponds
# to `muLog`, `sigmaLog` and `gamma`.

# %%
# The user can use `getParameterDescription()` method to access to the parameters of the auxiliary distribution.

# %%
ot.RandomGenerator.SetSeed(0)
marginR = ot.LogNormalMuSigma().getDistribution()
marginF = ot.Normal()
auxiliaryMarginals = [marginR, marginF]
auxiliaryDistribution = ot.JointDistribution(auxiliaryMarginals)
# Definition of parameters to be optimized
activeParameters = ot.Indices(5)
activeParameters.fill()
# WARNING : native parameters of distribution have to be considered
bounds = ot.Interval([14, 0.01, 0.0, 500, 20], [16, 0.2, 0.1, 1000, 70])
initialParameters = distribution.getParameter()
desc = auxiliaryDistribution.getParameterDescription()
p = auxiliaryDistribution.getParameter()
print(
    "Parameters of the auxiliary distribution:",
    ", ".join([f"{param}: {value:.3f}" for param, value in zip(desc, p)]),
)

physicalSpaceIS1 = ot.PhysicalSpaceCrossEntropyImportanceSampling(
    event, auxiliaryDistribution, activeParameters, initialParameters, bounds
)

# %%
# Custom optimization algorithm can be also specified for the auxiliary distribution parameters optimization, here for example we choose :class:`~openturns.TNC`.

# %%
physicalSpaceIS1.setOptimizationAlgorithm(ot.TNC())

# %%
# The number of samples per step can also be specified.

# %%
physicalSpaceIS1.setMaximumOuterSampling(1000)

# %%
# Finally, we run the algorithm and get the results.

# %%
physicalSpaceIS1.run()
physicalSpaceISResult1 = physicalSpaceIS1.getResult()
print("Probability of failure:", physicalSpaceISResult1.getProbabilityEstimate())
print("Coefficient of variation:", physicalSpaceISResult1.getCoefficientOfVariation())

# %%
# We can also decide to optimize only the means of the marginals and keep the other parameters identical to the initial distribution.

# %%
ot.RandomGenerator.SetSeed(0)
marginR = ot.LogNormalMuSigma(3e6, 3e5, 0.0).getDistribution()
marginF = ot.Normal(750.0, 50.0)
auxiliaryMarginals = [marginR, marginF]
auxiliaryDistribution = ot.JointDistribution(auxiliaryMarginals)
print("Parameters of initial distribution", auxiliaryDistribution.getParameter())

# Definition of parameters to be optimized
activeParameters = ot.Indices([0, 3])
# WARNING : native parameters of distribution have to be considered
bounds = ot.Interval([14, 500], [16, 1000])
initialParameters = [15, 750]
physicalSpaceIS2 = ot.PhysicalSpaceCrossEntropyImportanceSampling(
    event, auxiliaryDistribution, activeParameters, initialParameters, bounds
)
physicalSpaceIS2.run()
physicalSpaceISResult2 = physicalSpaceIS2.getResult()
print("Probability of failure:", physicalSpaceISResult2.getProbabilityEstimate())
print("Coefficient of variation:", physicalSpaceISResult2.getCoefficientOfVariation())

# %%
# Let us analyze the auxiliary output samples for the two previous simulations.
# We can plot initial (in blue) and auxiliary samples in physical space (orange
# for the first simulation and black for the second simulation).

# %%
graph = ot.Graph("Cloud of samples and failure domain", "R", "F")
graph.setLegendPosition("upper right")
auxiliarySamples1 = ot.Cloud(
    physicalSpaceISResult1.getAuxiliaryInputSample(),
    "orange",
    "fsquare",
    "Auxiliary samples, first case",
)
auxiliarySamples2 = ot.Cloud(
    physicalSpaceISResult2.getAuxiliaryInputSample(),
    "black",
    "bullet",
    "Auxiliary samples, second case",
)
graph.add(initialSamples)
graph.add(auxiliarySamples1)
graph.add(auxiliarySamples2)
graph.add(mycontour)
view = otv.View(graph)

# %%
# By analyzing the failure samples, one may want to include correlation parameters in the auxiliary distribution.
# In this last example, we add a Normal copula. The correlation parameter will be optimized with associated interval between 0 and 1.

# %%
ot.RandomGenerator.SetSeed(0)
marginR = ot.LogNormalMuSigma(3e6, 3e5, 0.0).getDistribution()
marginF = ot.Normal(750.0, 50.0)
auxiliaryMarginals = [marginR, marginF]
copula = ot.NormalCopula()
auxiliaryDistribution = ot.JointDistribution(auxiliaryMarginals, copula)
desc = auxiliaryDistribution.getParameterDescription()
p = auxiliaryDistribution.getParameter()
print(
    "Initial parameters of the auxiliary distribution:",
    ", ".join([f"{param}: {value:.3f}" for param, value in zip(desc, p)]),
)

# Definition of parameters to be optimized
activeParameters = ot.Indices(6)
activeParameters.fill()

bounds = ot.Interval(
    [14, 0.01, 0.0, 500.0, 20.0, 0.0], [16, 0.2, 0.1, 1000.0, 70.0, 1.0]
)
initialParameters = auxiliaryDistribution.getParameter()

physicalSpaceIS3 = ot.PhysicalSpaceCrossEntropyImportanceSampling(
    event, auxiliaryDistribution, activeParameters, initialParameters, bounds
)
physicalSpaceIS3.run()
physicalSpaceISResult3 = physicalSpaceIS3.getResult()
desc = physicalSpaceISResult3.getAuxiliaryDistribution().getParameterDescription()
p = physicalSpaceISResult3.getAuxiliaryDistribution().getParameter()
print(
    "Optimized parameters of the auxiliary distribution:",
    ", ".join([f"{param}: {value:.3f}" for param, value in zip(desc, p)]),
)
print("Probability of failure: ", physicalSpaceISResult3.getProbabilityEstimate())
print("Coefficient of variation: ", physicalSpaceISResult3.getCoefficientOfVariation())

# %%
# Finally, we plot the new auxiliary samples in black.

# %%
graph = ot.Graph("Cloud of samples and failure domain", "R", "F")
graph.setLegendPosition("upper right")
auxiliarySamples1 = ot.Cloud(
    physicalSpaceISResult1.getAuxiliaryInputSample(),
    "orange",
    "fsquare",
    "Auxiliary samples, first case",
)
auxiliarySamples3 = ot.Cloud(
    physicalSpaceISResult3.getAuxiliaryInputSample(),
    "black",
    "bullet",
    "Auxiliary samples, second case",
)
graph.add(initialSamples)
graph.add(auxiliarySamples1)
graph.add(auxiliarySamples3)
graph.add(mycontour)

# sphinx_gallery_thumbnail_number = 4
view = otv.View(graph)

# %%
# The `quantileLevel` parameter can be also changed using the :class:`~openturns.ResourceMap` key : `CrossEntropyImportanceSampling-DefaultQuantileLevel`.
# Be careful that this key changes the value number of both :class:`~openturns.StandardSpaceCrossEntropyImportanceSampling`
# and :class:`~openturns.PhysicalSpaceCrossEntropyImportanceSampling`.

# %%
ot.ResourceMap.SetAsScalar("CrossEntropyImportanceSampling-DefaultQuantileLevel", 0.4)
physicalSpaceIS4 = ot.PhysicalSpaceCrossEntropyImportanceSampling(
    event, auxiliaryDistribution, activeParameters, initialParameters, bounds
)
print("Modified quantile level:", physicalSpaceIS4.getQuantileLevel())


# %%
# The optimized auxiliary distribution with a dependency between the two marginals allows one to better fit the failure domain, resulting in a lower coefficient of variation.

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal
from openturns.usecases.fire_satellite import FireSatelliteModel


ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(5)

"""
Test the import of the FireSatellite model data class.
"""
m = FireSatelliteModel()

assert_almost_equal(m.model(m.distribution.getMean())[0], 0.011659, 1e-2)

# test getMarginal (if serialization is available)
if ot.PlatformInfo.HasFeature("libxml2"):
    m0 = m.model.getMarginal(0)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

size = 100
xuniform = ot.Uniform(0.9, 1.1)
x = xuniform.getSample(size)
yuniform = ot.Uniform(1.9, 2.1)
y = yuniform.getSample(size)
w = [1.0] * size
f = ot.SymbolicFunction(["x"], ["2.0*x"])
coll = []
coll.append(f)
basis = ot.Basis(coll)
indices = list(range(len(coll)))

fittingAlgo = ot.CorrectedLeaveOneOut()

print("algo =", fittingAlgo)

result = fittingAlgo.run(x, y, w, basis, indices)

print("result = %g" % result)

# illustrate the other usages
proxy = ot.DesignProxy(x, basis)
method = ot.QRMethod(proxy, indices)
# use the other run methods to cover them
result2 = fittingAlgo.run(y, w, indices, proxy)
ott.assert_almost_equal(result, result2, 1e-5, 1e-5)
result3 = fittingAlgo.run(y, indices, proxy)
ott.assert_almost_equal(result, result3, 1e-5, 1e-5)
result4 = fittingAlgo.run(method, y)
ott.assert_almost_equal(result, result4, 1e-5, 1e-5)



#! /usr/bin/env python

import openturns as ot


ot.TESTPREAMBLE()


# We create a numerical math function
myFunction = ot.SymbolicFunction(["E", "F", "L", "I"], ["-F*L^3/(3*E*I)"])

dim = myFunction.getInputDimension()

# We create a normal distribution point of dimension 1
mean = [50.0, 1.0, 10.0, 5.0]
sigma = [1.0] * dim
R = ot.IdentityMatrix(dim)
myDistribution = ot.Normal(mean, sigma, R)

# We create a 'usual' RandomVector from the Distribution
vect = ot.RandomVector(myDistribution)

# We create a composite random vector
output = ot.CompositeRandomVector(myFunction, vect)

# We create an Event from this RandomVector
myEvent = ot.ThresholdEvent(output, ot.Less(), -1.5)

# We create a NearestPoint algorithm
myAbdoRackwitz = ot.AbdoRackwitz()
myAbdoRackwitz.setStartingPoint(mean)
myAbdoRackwitz.setMaximumIterationNumber(100)
myAbdoRackwitz.setMaximumAbsoluteError(1.0e-10)
myAbdoRackwitz.setMaximumRelativeError(1.0e-10)
myAbdoRackwitz.setMaximumResidualError(1.0e-10)
myAbdoRackwitz.setMaximumConstraintError(1.0e-10)
print("myAbdoRackwitz=", myAbdoRackwitz)

# We create a FORM algorithm
# The first parameter is an OptimizationAlgorithm
# The second parameter is an event
# The third parameter is a starting point for the design point research
myAlgo = ot.SORM(myAbdoRackwitz, myEvent)

# Perform the simulation
myAlgo.run()

# Stream out the result
result = myAlgo.getResult()
digits = 5
print("Breitung event probability=%.6f" % result.getEventProbabilityBreitung())
print(
    "Breitung generalized reliability index=%.6f"
    % result.getGeneralisedReliabilityIndexBreitung()
)
print("Hohenbichler event probability=%.6f" % result.getEventProbabilityHohenbichler())
print(
    "Hohenbichler generalized reliability index=%.6f"
    % result.getGeneralisedReliabilityIndexHohenbichler()
)
print("Tvedt event probability=%.6f" % result.getEventProbabilityTvedt())
print(
    "Tvedt generalized reliability index=%.6f"
    % result.getGeneralisedReliabilityIndexTvedt()
)
curvatures = result.getSortedCurvatures()
curvatures[2] = 0.0
print("sorted curvatures=", curvatures)
print("standard space design point=", result.getStandardSpaceDesignPoint())
print("physical space design point=", result.getPhysicalSpaceDesignPoint())

# Is the standard point origin in failure space?
print(
    "is standard point origin in failure space? ",
    result.getIsStandardPointOriginInFailureSpace(),
)

print("importance factors=", result.getImportanceFactors())
print("Hasofer reliability index=%.6f" % result.getHasoferReliabilityIndex())



"""
Use case : viscous free fall
============================
"""

import openturns as ot
import numpy as np


def AltiFunc(X):
    g = 9.81
    z0 = X[0]
    v0 = X[1]
    m = X[2]
    c = X[3]
    tau = m / c
    vinf = -m * g / c
    t = np.array(AltiFunc.vertices)
    z = z0 + vinf * t + tau * (v0 - vinf) * (1 - np.exp(-t / tau))
    z = np.maximum(z, 0.0)
    return [[zeta[0]] for zeta in z]


class ViscousFreeFall:
    """
    Data class for the viscous free fall.


    Attributes
    ----------

    dim : int
        The dimension of the problem, dim=4.

    outputDimension : int
        The output dimension of the problem, outputDimension=1.

    tmin : float
        Minimum time, tmin = 0.0

    tmax : float
        Maximum time, tmax = 12.0

    gridsize : int
        Number of time steps, gridsize = 100.

    mesh : :class:`~openturns.Mesh`
        Time grid

    vertices : :class:`~openturns.Sample`
        Vertices of the mesh

    distZ0 : :class:`~openturns.Uniform`
        Distribution of the initial altitude: `Uniform(100.0, 150.0)`

    distV0 : :class:`~openturns.Normal`
        Distribution of the initial speed: `Normal(55.0, 10.0)`

    distM : :class:`~openturns.Normal`
        Distribution of the mass: `Normal(80.0, 8.0)`

    distC : :class:`~openturns.Uniform`
        Distribution of the drag: `Uniform(0.0, 30.0)`

    distribution : :class:`~openturns.JointDistribution`
        The joint distribution of the input parameters.

    model : :class:`~openturns.PythonPointToFieldFunction`
        The exact solution of the fall


    Examples
    --------
    >>> from openturns.usecases import viscous_free_fall
    >>> # Load the viscous free fall example
    >>> vff = viscous_free_fall.ViscousFreeFall()
    """

    def __init__(self):
        self.dim = 4  # number of inputs
        self.outputDimension = 1  # dimension of the output

        self.tmin = 0.0  # Minimum time
        self.tmax = 12.0  # Maximum time
        self.gridsize = 100  # Number of time steps
        self.mesh = ot.IntervalMesher([self.gridsize - 1]).build(
            ot.Interval(self.tmin, self.tmax)
        )
        AltiFunc.vertices = self.mesh.getVertices()
        self.vertices = self.mesh.getVertices()

        # Marginals
        self.distZ0 = ot.Uniform(100.0, 150.0)
        self.distV0 = ot.Normal(55.0, 10.0)
        self.distM = ot.Normal(80.0, 8.0)
        self.distC = ot.Uniform(0.0, 30.0)

        # Joint distribution
        self.distribution = ot.JointDistribution(
            [self.distZ0, self.distV0, self.distM, self.distC]
        )
        description = ["z0", "v0", "m", "c"]
        self.distribution.setDescription(description)

        # Exact solution
        self.model = ot.PythonPointToFieldFunction(
            self.dim, self.mesh, self.outputDimension, AltiFunc
        )
        self.model.setInputDescription(description)
        self.model.setOutputDescription(["z"])
        self.alti = self.model  # deprecated



"""
Manipulate a time series
========================
"""

# %%
# The objective here is to create and manipulate a time series.
# A time series is a particular field where the mesh :math:`\mathcal{M}` 1-d and regular, eg a time grid :math:`(t_0, \dots, t_{N-1})`.
#
# It is possible to draw a time series, using interpolation between the values: see the use case on the Field.
#
# A time series can be obtained as a realization of a multivariate stochastic process
# :math:`X: \Omega \times [0,T] \rightarrow \mathbb{R}^d` of dimension :math:`d` where :math:`[0,T]`
# is discretized according to the regular grid :math:`(t_0, \dots, t_{N-1})`.
# The  values :math:`(\vect{x}_0, \dots, \vect{x}_{N-1})` of the  time series are defined by:
#
# .. math::
#    \forall i \in [0, N-1],\quad \vect{x}_i= X(\omega)(t_i)
#
#
# A time series is stored in the :class:`~openturns.TimeSeries` object that stores the regular time grid and the associated values.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Create the RegularGrid
tMin = 0.0
timeStep = 0.1
N = 100
myTimeGrid = ot.RegularGrid(tMin, timeStep, N)

# %%
# Case 1: Create a time series from a time grid and values.
# Be careful that the number of steps of the time grid must correspond to the size of the values
myValues = ot.Normal(3).getSample(myTimeGrid.getVertices().getSize())
myTimeSeries = ot.TimeSeries(myTimeGrid, myValues)
myTimeSeries

# %%
# Case 2: Get a time series from a Process
myProcess = ot.WhiteNoise(ot.Normal(3), myTimeGrid)
myTimeSeries2 = myProcess.getRealization()
myTimeSeries2

# %%
# Get the number of values of the time series
myTimeSeries.getSize()

# %%
# Get the dimension of the values observed at each time
myTimeSeries.getMesh().getDimension()

# %%
# Get the value :math:`X_i` at index :math:`i`
i = 37
print("Xi = ", myTimeSeries.getValueAtIndex(i))

# %%
# Get the time series at index :math:`i` : :math:`X_i`
i = 37
print("Xi = ", myTimeSeries[i])

# %%
# Get a the marginal value at index :math:`i` of the time series
i = 37
# get the time stamp:
print("ti = ", myTimeSeries.getTimeGrid().getValue(i))
# get the first component of the corresponding value :
print("Xi1 = ", myTimeSeries[i, 0])

# %%
# Get all the values :math:`(X_1, \dots, X_n)` of the time series
myTimeSeries.getValues()

# %%
# Compute the temporal Mean
# It corresponds to the mean of the values of the time series
myTimeSeries.getInputMean()

# %%
# Draw the marginal :math:`i` of the time series using linear interpolation
graph = myTimeSeries.drawMarginal(0)
view = otv.View(graph)

# %%
# With no interpolation
graph = myTimeSeries.drawMarginal(0, False)
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()
